@@ -2,7 +2,7 @@
 //
 //  See http://www.boost.org for most recent version, including documentation.
 //
-//  Copyright Antony Polukhin, 2013-2023.
+//  Copyright Antony Polukhin, 2013-2025.
 //
 //  Distributed under the Boost
 //  Software License, Version 1.0. (See accompanying file
@@ -13,26 +13,10 @@
 #include <utility>
 
 #include <boost/any.hpp>
-#include "test.hpp"
-#include <boost/move/move.hpp>
-
-#ifdef BOOST_NO_CXX11_RVALUE_REFERENCES
-
-int main()
-{
-    BOOST_STATIC_ASSERT(false);
-    return EXIT_SUCCESS;
-}
-
-#else
 
 
-int main()
-{
+int main() {
     int i = boost::any_cast<int&>(10);
     (void)i;
-    return EXIT_SUCCESS;
 }
 
-#endif
-
