@@ -2,7 +2,7 @@
 // thread_pool.hpp
 // ~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2025 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -29,11 +29,11 @@ namespace asio {
 namespace detail {
   struct thread_pool_bits
   {
-    BOOST_ASIO_STATIC_CONSTEXPR(unsigned int, blocking_never = 1);
-    BOOST_ASIO_STATIC_CONSTEXPR(unsigned int, blocking_always = 2);
-    BOOST_ASIO_STATIC_CONSTEXPR(unsigned int, blocking_mask = 3);
-    BOOST_ASIO_STATIC_CONSTEXPR(unsigned int, relationship_continuation = 4);
-    BOOST_ASIO_STATIC_CONSTEXPR(unsigned int, outstanding_work_tracked = 8);
+    static constexpr unsigned int blocking_never = 1;
+    static constexpr unsigned int blocking_always = 2;
+    static constexpr unsigned int blocking_mask = 3;
+    static constexpr unsigned int relationship_continuation = 4;
+    static constexpr unsigned int outstanding_work_tracked = 8;
   };
 } // namespace detail
 
@@ -42,6 +42,17 @@ namespace detail {
  * The thread pool class is an execution context where functions are permitted
  * to run on one of a fixed number of threads.
  *
+ * @par Thread Safety
+ * @e Distinct @e objects: Safe.@n
+ * @e Shared @e objects: Safe, with the specific exceptions of the join()
+ * wait() and notify_fork() functions. The join() and wait() functions must not
+ * be called at the same time as other calls to join() or wait() on the same
+ * pool. The notify_fork() function should not be called while any thread_pool
+ * function, or any function on an I/O object that is associated with the
+ * thread_pool, is being called in another thread. (In effect, this means that
+ * notify_fork() is safe only on a thread pool that has no internal or attached
+ * threads at the time.)
+ *
  * @par Submitting tasks to the pool
  *
  * To submit functions to the thread pool, use the @ref boost::asio::dispatch,
@@ -85,9 +96,6 @@ public:
   /// Executor used to submit functions to a thread pool.
   typedef basic_executor_type<std::allocator<void>, 0> executor_type;
 
-  /// Scheduler used to schedule receivers on a thread pool.
-  typedef basic_executor_type<std::allocator<void>, 0> scheduler_type;
-
 #if !defined(BOOST_ASIO_NO_TS_EXECUTORS)
   /// Constructs a pool with an automatically determined number of threads.
   BOOST_ASIO_DECL thread_pool();
@@ -96,6 +104,19 @@ public:
   /// Constructs a pool with a specified number of threads.
   BOOST_ASIO_DECL thread_pool(std::size_t num_threads);
 
+  /// Constructs a pool with a specified number of threads.
+  /**
+   * Construct with a service maker, to create an initial set of services that
+   * will be installed into the execution context at construction time.
+   *
+   * @param num_threads The number of threads required.
+   *
+   * @param initial_services Used to create the initial services. The @c make
+   * function will be called once at the end of execution_context construction.
+   */
+  BOOST_ASIO_DECL thread_pool(std::size_t num_threads,
+      const execution_context::service_maker& initial_services);
+
   /// Destructor.
   /**
    * Automatically stops and joins the pool, if not explicitly done beforehand.
@@ -103,13 +124,10 @@ public:
   BOOST_ASIO_DECL ~thread_pool();
 
   /// Obtains the executor associated with the pool.
-  executor_type get_executor() BOOST_ASIO_NOEXCEPT;
+  executor_type get_executor() noexcept;
 
   /// Obtains the executor associated with the pool.
-  executor_type executor() BOOST_ASIO_NOEXCEPT;
-
-  /// Obtains the scheduler associated with the pool.
-  scheduler_type scheduler() BOOST_ASIO_NOEXCEPT;
+  executor_type executor() noexcept;
 
   /// Stops the threads.
   /**
@@ -139,12 +157,14 @@ public:
    * This function blocks until the threads in the pool have completed. If @c
    * stop() is not called prior to @c wait(), the @c wait() call will wait
    * until the pool has no more outstanding work.
+   *
+   * @note @c wait() is synonymous with @c join().
    */
   BOOST_ASIO_DECL void wait();
 
 private:
-  thread_pool(const thread_pool&) BOOST_ASIO_DELETED;
-  thread_pool& operator=(const thread_pool&) BOOST_ASIO_DELETED;
+  thread_pool(const thread_pool&) = delete;
+  thread_pool& operator=(const thread_pool&) = delete;
 
   struct thread_function;
 
@@ -159,6 +179,9 @@ private:
 
   // The current number of threads in the pool.
   detail::atomic_count num_threads_;
+
+  // Whether a join call will have any effect.
+  bool joinable_;
 };
 
 /// Executor implementation type used to submit functions to a thread pool.
@@ -166,34 +189,8 @@ template <typename Allocator, unsigned int Bits>
 class thread_pool::basic_executor_type : detail::thread_pool_bits
 {
 public:
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-  /// (Deprecated.) The sender type, when this type is used as a scheduler.
-  typedef basic_executor_type sender_type;
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-
-  /// The bulk execution shape type.
-  typedef std::size_t shape_type;
-
-  /// The bulk execution index type.
-  typedef std::size_t index_type;
-
-#if defined(BOOST_ASIO_HAS_DEDUCED_EXECUTION_IS_TYPED_SENDER_TRAIT) \
-  && defined(BOOST_ASIO_HAS_STD_EXCEPTION_PTR)
-  template <
-      template <typename...> class Tuple,
-      template <typename...> class Variant>
-  using value_types = Variant<Tuple<>>;
-
-  template <template <typename...> class Variant>
-  using error_types = Variant<std::exception_ptr>;
-
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, sends_done = true);
-#endif // defined(BOOST_ASIO_HAS_DEDUCED_EXECUTION_IS_TYPED_SENDER_TRAIT)
-       //   && defined(BOOST_ASIO_HAS_STD_EXCEPTION_PTR)
-
   /// Copy constructor.
-  basic_executor_type(
-      const basic_executor_type& other) BOOST_ASIO_NOEXCEPT
+  basic_executor_type(const basic_executor_type& other) noexcept
     : pool_(other.pool_),
       allocator_(other.allocator_),
       bits_(other.bits_)
@@ -203,20 +200,18 @@ public:
         pool_->scheduler_.work_started();
   }
 
-#if defined(BOOST_ASIO_HAS_MOVE) || defined(GENERATING_DOCUMENTATION)
   /// Move constructor.
-  basic_executor_type(basic_executor_type&& other) BOOST_ASIO_NOEXCEPT
+  basic_executor_type(basic_executor_type&& other) noexcept
     : pool_(other.pool_),
-      allocator_(BOOST_ASIO_MOVE_CAST(Allocator)(other.allocator_)),
+      allocator_(static_cast<Allocator&&>(other.allocator_)),
       bits_(other.bits_)
   {
     if (Bits & outstanding_work_tracked)
       other.pool_ = 0;
   }
-#endif // defined(BOOST_ASIO_HAS_MOVE) || defined(GENERATING_DOCUMENTATION)
 
   /// Destructor.
-  ~basic_executor_type() BOOST_ASIO_NOEXCEPT
+  ~basic_executor_type() noexcept
   {
     if (Bits & outstanding_work_tracked)
       if (pool_)
@@ -224,14 +219,10 @@ public:
   }
 
   /// Assignment operator.
-  basic_executor_type& operator=(
-      const basic_executor_type& other) BOOST_ASIO_NOEXCEPT;
+  basic_executor_type& operator=(const basic_executor_type& other) noexcept;
 
-#if defined(BOOST_ASIO_HAS_MOVE) || defined(GENERATING_DOCUMENTATION)
   /// Move assignment operator.
-  basic_executor_type& operator=(
-      basic_executor_type&& other) BOOST_ASIO_NOEXCEPT;
-#endif // defined(BOOST_ASIO_HAS_MOVE) || defined(GENERATING_DOCUMENTATION)
+  basic_executor_type& operator=(basic_executor_type&& other) noexcept;
 
 #if !defined(GENERATING_DOCUMENTATION)
 private:
@@ -249,7 +240,7 @@ private:
    * auto ex2 = boost::asio::require(ex1,
    *     boost::asio::execution::blocking.possibly); @endcode
    */
-  BOOST_ASIO_CONSTEXPR basic_executor_type<Allocator,
+  constexpr basic_executor_type<Allocator,
       BOOST_ASIO_UNSPECIFIED(Bits & ~blocking_mask)>
   require(execution::blocking_t::possibly_t) const
   {
@@ -267,7 +258,7 @@ private:
    * auto ex2 = boost::asio::require(ex1,
    *     boost::asio::execution::blocking.always); @endcode
    */
-  BOOST_ASIO_CONSTEXPR basic_executor_type<Allocator,
+  constexpr basic_executor_type<Allocator,
       BOOST_ASIO_UNSPECIFIED((Bits & ~blocking_mask) | blocking_always)>
   require(execution::blocking_t::always_t) const
   {
@@ -286,7 +277,7 @@ private:
    * auto ex2 = boost::asio::require(ex1,
    *     boost::asio::execution::blocking.never); @endcode
    */
-  BOOST_ASIO_CONSTEXPR basic_executor_type<Allocator,
+  constexpr basic_executor_type<Allocator,
       BOOST_ASIO_UNSPECIFIED(Bits & ~blocking_mask)>
   require(execution::blocking_t::never_t) const
   {
@@ -304,8 +295,7 @@ private:
    * auto ex2 = boost::asio::require(ex1,
    *     boost::asio::execution::relationship.fork); @endcode
    */
-  BOOST_ASIO_CONSTEXPR basic_executor_type require(
-      execution::relationship_t::fork_t) const
+  constexpr basic_executor_type require(execution::relationship_t::fork_t) const
   {
     return basic_executor_type(pool_,
         allocator_, bits_ & ~relationship_continuation);
@@ -321,7 +311,7 @@ private:
    * auto ex2 = boost::asio::require(ex1,
    *     boost::asio::execution::relationship.continuation); @endcode
    */
-  BOOST_ASIO_CONSTEXPR basic_executor_type require(
+  constexpr basic_executor_type require(
       execution::relationship_t::continuation_t) const
   {
     return basic_executor_type(pool_,
@@ -338,7 +328,7 @@ private:
    * auto ex2 = boost::asio::require(ex1,
    *     boost::asio::execution::outstanding_work.tracked); @endcode
    */
-  BOOST_ASIO_CONSTEXPR basic_executor_type<Allocator,
+  constexpr basic_executor_type<Allocator,
       BOOST_ASIO_UNSPECIFIED(Bits | outstanding_work_tracked)>
   require(execution::outstanding_work_t::tracked_t) const
   {
@@ -356,7 +346,7 @@ private:
    * auto ex2 = boost::asio::require(ex1,
    *     boost::asio::execution::outstanding_work.untracked); @endcode
    */
-  BOOST_ASIO_CONSTEXPR basic_executor_type<Allocator,
+  constexpr basic_executor_type<Allocator,
       BOOST_ASIO_UNSPECIFIED(Bits & ~outstanding_work_tracked)>
   require(execution::outstanding_work_t::untracked_t) const
   {
@@ -375,7 +365,7 @@ private:
    *     boost::asio::execution::allocator(my_allocator)); @endcode
    */
   template <typename OtherAllocator>
-  BOOST_ASIO_CONSTEXPR basic_executor_type<OtherAllocator, Bits>
+  constexpr basic_executor_type<OtherAllocator, Bits>
   require(execution::allocator_t<OtherAllocator> a) const
   {
     return basic_executor_type<OtherAllocator, Bits>(
@@ -392,7 +382,7 @@ private:
    * auto ex2 = boost::asio::require(ex1,
    *     boost::asio::execution::allocator); @endcode
    */
-  BOOST_ASIO_CONSTEXPR basic_executor_type<std::allocator<void>, Bits>
+  constexpr basic_executor_type<std::allocator<void>, Bits>
   require(execution::allocator_t<void>) const
   {
     return basic_executor_type<std::allocator<void>, Bits>(
@@ -406,25 +396,6 @@ private:
   friend struct boost::asio::execution::detail::outstanding_work_t<0>;
 #endif // !defined(GENERATING_DOCUMENTATION)
 
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-  /// (Deprecated.) Query the current value of the @c bulk_guarantee property.
-  /**
-   * Do not call this function directly. It is intended for use with the
-   * boost::asio::query customisation point.
-   *
-   * For example:
-   * @code auto ex = my_thread_pool.executor();
-   * if (boost::asio::query(ex, boost::asio::execution::bulk_guarantee)
-   *       == boost::asio::execution::bulk_guarantee.parallel)
-   *   ... @endcode
-   */
-  static BOOST_ASIO_CONSTEXPR execution::bulk_guarantee_t query(
-      execution::bulk_guarantee_t) BOOST_ASIO_NOEXCEPT
-  {
-    return execution::bulk_guarantee.parallel;
-  }
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-
   /// Query the current value of the @c mapping property.
   /**
    * Do not call this function directly. It is intended for use with the
@@ -436,8 +407,7 @@ private:
    *       == boost::asio::execution::mapping.thread)
    *   ... @endcode
    */
-  static BOOST_ASIO_CONSTEXPR execution::mapping_t query(
-      execution::mapping_t) BOOST_ASIO_NOEXCEPT
+  static constexpr execution::mapping_t query(execution::mapping_t) noexcept
   {
     return execution::mapping.thread;
   }
@@ -452,7 +422,7 @@ private:
    * boost::asio::thread_pool& pool = boost::asio::query(
    *     ex, boost::asio::execution::context); @endcode
    */
-  thread_pool& query(execution::context_t) const BOOST_ASIO_NOEXCEPT
+  thread_pool& query(execution::context_t) const noexcept
   {
     return *pool_;
   }
@@ -468,8 +438,7 @@ private:
    *       == boost::asio::execution::blocking.always)
    *   ... @endcode
    */
-  BOOST_ASIO_CONSTEXPR execution::blocking_t query(
-      execution::blocking_t) const BOOST_ASIO_NOEXCEPT
+  constexpr execution::blocking_t query(execution::blocking_t) const noexcept
   {
     return (bits_ & blocking_never)
       ? execution::blocking_t(execution::blocking.never)
@@ -489,8 +458,8 @@ private:
    *       == boost::asio::execution::relationship.continuation)
    *   ... @endcode
    */
-  BOOST_ASIO_CONSTEXPR execution::relationship_t query(
-      execution::relationship_t) const BOOST_ASIO_NOEXCEPT
+  constexpr execution::relationship_t query(
+      execution::relationship_t) const noexcept
   {
     return (bits_ & relationship_continuation)
       ? execution::relationship_t(execution::relationship.continuation)
@@ -508,8 +477,8 @@ private:
    *       == boost::asio::execution::outstanding_work.tracked)
    *   ... @endcode
    */
-  static BOOST_ASIO_CONSTEXPR execution::outstanding_work_t query(
-      execution::outstanding_work_t) BOOST_ASIO_NOEXCEPT
+  static constexpr execution::outstanding_work_t query(
+      execution::outstanding_work_t) noexcept
   {
     return (Bits & outstanding_work_tracked)
       ? execution::outstanding_work_t(execution::outstanding_work.tracked)
@@ -527,8 +496,8 @@ private:
    *     boost::asio::execution::allocator); @endcode
    */
   template <typename OtherAllocator>
-  BOOST_ASIO_CONSTEXPR Allocator query(
-      execution::allocator_t<OtherAllocator>) const BOOST_ASIO_NOEXCEPT
+  constexpr Allocator query(
+      execution::allocator_t<OtherAllocator>) const noexcept
   {
     return allocator_;
   }
@@ -543,8 +512,7 @@ private:
    * auto alloc = boost::asio::query(ex,
    *     boost::asio::execution::allocator); @endcode
    */
-  BOOST_ASIO_CONSTEXPR Allocator query(
-      execution::allocator_t<void>) const BOOST_ASIO_NOEXCEPT
+  constexpr Allocator query(execution::allocator_t<void>) const noexcept
   {
     return allocator_;
   }
@@ -559,7 +527,7 @@ private:
    * std::size_t occupancy = boost::asio::query(
    *     ex, boost::asio::execution::occupancy); @endcode
    */
-  std::size_t query(execution::occupancy_t) const BOOST_ASIO_NOEXCEPT
+  std::size_t query(execution::occupancy_t) const noexcept
   {
     return static_cast<std::size_t>(pool_->num_threads_);
   }
@@ -570,14 +538,14 @@ public:
    * @return @c true if the current thread is running the thread pool. Otherwise
    * returns @c false.
    */
-  bool running_in_this_thread() const BOOST_ASIO_NOEXCEPT;
+  bool running_in_this_thread() const noexcept;
 
   /// Compare two executors for equality.
   /**
    * Two executors are equal if they refer to the same underlying thread pool.
    */
   friend bool operator==(const basic_executor_type& a,
-      const basic_executor_type& b) BOOST_ASIO_NOEXCEPT
+      const basic_executor_type& b) noexcept
   {
     return a.pool_ == b.pool_
       && a.allocator_ == b.allocator_
@@ -589,7 +557,7 @@ public:
    * Two executors are equal if they refer to the same underlying thread pool.
    */
   friend bool operator!=(const basic_executor_type& a,
-      const basic_executor_type& b) BOOST_ASIO_NOEXCEPT
+      const basic_executor_type& b) noexcept
   {
     return a.pool_ != b.pool_
       || a.allocator_ != b.allocator_
@@ -598,58 +566,16 @@ public:
 
   /// Execution function.
   template <typename Function>
-  void execute(BOOST_ASIO_MOVE_ARG(Function) f) const
+  void execute(Function&& f) const
   {
-    this->do_execute(BOOST_ASIO_MOVE_CAST(Function)(f),
+    this->do_execute(static_cast<Function&&>(f),
         integral_constant<bool, (Bits & blocking_always) != 0>());
   }
 
 public:
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-  /// (Deprecated.) Bulk execution function.
-  template <typename Function>
-  void bulk_execute(BOOST_ASIO_MOVE_ARG(Function) f, std::size_t n) const
-  {
-    this->do_bulk_execute(BOOST_ASIO_MOVE_CAST(Function)(f), n,
-        integral_constant<bool, (Bits & blocking_always) != 0>());
-  }
-
-  /// (Deprecated.) Schedule function.
-  /**
-   * Do not call this function directly. It is intended for use with the
-   * execution::schedule customisation point.
-   *
-   * @return An object that satisfies the sender concept.
-   */
-  sender_type schedule() const BOOST_ASIO_NOEXCEPT
-  {
-    return *this;
-  }
-
-  /// (Deprecated.) Connect function.
-  /**
-   * Do not call this function directly. It is intended for use with the
-   * execution::connect customisation point.
-   *
-   * @return An object of an unspecified type that satisfies the @c
-   * operation_state concept.
-   */
-  template <BOOST_ASIO_EXECUTION_RECEIVER_OF_0 Receiver>
-#if defined(GENERATING_DOCUMENTATION)
-  unspecified
-#else // defined(GENERATING_DOCUMENTATION)
-  execution::detail::as_operation<basic_executor_type, Receiver>
-#endif // defined(GENERATING_DOCUMENTATION)
-  connect(BOOST_ASIO_MOVE_ARG(Receiver) r) const
-  {
-    return execution::detail::as_operation<basic_executor_type, Receiver>(
-        *this, BOOST_ASIO_MOVE_CAST(Receiver)(r));
-  }
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-
 #if !defined(BOOST_ASIO_NO_TS_EXECUTORS)
   /// Obtain the underlying execution context.
-  thread_pool& context() const BOOST_ASIO_NOEXCEPT;
+  thread_pool& context() const noexcept;
 
   /// Inform the thread pool that it has some outstanding work to do.
   /**
@@ -657,7 +583,7 @@ public:
    * This ensures that the thread pool's join() function will not return while
    * the work is underway.
    */
-  void on_work_started() const BOOST_ASIO_NOEXCEPT;
+  void on_work_started() const noexcept;
 
   /// Inform the thread pool that some work is no longer outstanding.
   /**
@@ -665,7 +591,7 @@ public:
    * finished. Once the count of unfinished work reaches zero, the thread
    * pool's join() function is permitted to exit.
    */
-  void on_work_finished() const BOOST_ASIO_NOEXCEPT;
+  void on_work_finished() const noexcept;
 
   /// Request the thread pool to invoke the given function object.
   /**
@@ -682,8 +608,7 @@ public:
    * internal storage needed for function invocation.
    */
   template <typename Function, typename OtherAllocator>
-  void dispatch(BOOST_ASIO_MOVE_ARG(Function) f,
-      const OtherAllocator& a) const;
+  void dispatch(Function&& f, const OtherAllocator& a) const;
 
   /// Request the thread pool to invoke the given function object.
   /**
@@ -699,8 +624,7 @@ public:
    * internal storage needed for function invocation.
    */
   template <typename Function, typename OtherAllocator>
-  void post(BOOST_ASIO_MOVE_ARG(Function) f,
-      const OtherAllocator& a) const;
+  void post(Function&& f, const OtherAllocator& a) const;
 
   /// Request the thread pool to invoke the given function object.
   /**
@@ -720,8 +644,7 @@ public:
    * internal storage needed for function invocation.
    */
   template <typename Function, typename OtherAllocator>
-  void defer(BOOST_ASIO_MOVE_ARG(Function) f,
-      const OtherAllocator& a) const;
+  void defer(Function&& f, const OtherAllocator& a) const;
 #endif // !defined(BOOST_ASIO_NO_TS_EXECUTORS)
 
 private:
@@ -729,7 +652,7 @@ private:
   template <typename, unsigned int> friend class basic_executor_type;
 
   // Constructor used by thread_pool::get_executor().
-  explicit basic_executor_type(thread_pool& p) BOOST_ASIO_NOEXCEPT
+  explicit basic_executor_type(thread_pool& p) noexcept
     : pool_(&p),
       allocator_(),
       bits_(0)
@@ -740,7 +663,7 @@ private:
 
   // Constructor used by require().
   basic_executor_type(thread_pool* p,
-      const Allocator& a, unsigned int bits) BOOST_ASIO_NOEXCEPT
+      const Allocator& a, unsigned int bits) noexcept
     : pool_(p),
       allocator_(a),
       bits_(bits)
@@ -752,21 +675,11 @@ private:
 
   /// Execution helper implementation for possibly and never blocking.
   template <typename Function>
-  void do_execute(BOOST_ASIO_MOVE_ARG(Function) f, false_type) const;
+  void do_execute(Function&& f, false_type) const;
 
   /// Execution helper implementation for always blocking.
   template <typename Function>
-  void do_execute(BOOST_ASIO_MOVE_ARG(Function) f, true_type) const;
-
-  /// Bulk execution helper implementation for possibly and never blocking.
-  template <typename Function>
-  void do_bulk_execute(BOOST_ASIO_MOVE_ARG(Function) f,
-      std::size_t n, false_type) const;
-
-  /// Bulk execution helper implementation for always blocking.
-  template <typename Function>
-  void do_bulk_execute(BOOST_ASIO_MOVE_ARG(Function) f,
-      std::size_t n, true_type) const;
+  void do_execute(Function&& f, true_type) const;
 
   // The underlying thread pool.
   thread_pool* pool_;
@@ -789,8 +702,8 @@ struct equality_comparable<
     boost::asio::thread_pool::basic_executor_type<Allocator, Bits>
   >
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
 };
 
 #endif // !defined(BOOST_ASIO_HAS_DEDUCED_EQUALITY_COMPARABLE_TRAIT)
@@ -803,53 +716,13 @@ struct execute_member<
     Function
   >
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = false);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = false;
   typedef void result_type;
 };
 
 #endif // !defined(BOOST_ASIO_HAS_DEDUCED_EXECUTE_MEMBER_TRAIT)
 
-#if !defined(BOOST_ASIO_HAS_DEDUCED_SCHEDULE_MEMBER_TRAIT)
-
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-
-template <typename Allocator, unsigned int Bits>
-struct schedule_member<
-    const boost::asio::thread_pool::basic_executor_type<Allocator, Bits>
-  >
-{
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = false);
-  typedef boost::asio::thread_pool::basic_executor_type<
-      Allocator, Bits> result_type;
-};
-
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-
-#endif // !defined(BOOST_ASIO_HAS_DEDUCED_SCHEDULE_MEMBER_TRAIT)
-
-#if !defined(BOOST_ASIO_HAS_DEDUCED_CONNECT_MEMBER_TRAIT)
-
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-
-template <typename Allocator, unsigned int Bits, typename Receiver>
-struct connect_member<
-    const boost::asio::thread_pool::basic_executor_type<Allocator, Bits>,
-    Receiver
-  >
-{
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = false);
-  typedef boost::asio::execution::detail::as_operation<
-      boost::asio::thread_pool::basic_executor_type<Allocator, Bits>,
-      Receiver> result_type;
-};
-
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-
-#endif // !defined(BOOST_ASIO_HAS_DEDUCED_CONNECT_MEMBER_TRAIT)
-
 #if !defined(BOOST_ASIO_HAS_DEDUCED_REQUIRE_MEMBER_TRAIT)
 
 template <typename Allocator, unsigned int Bits>
@@ -858,8 +731,8 @@ struct require_member<
     boost::asio::execution::blocking_t::possibly_t
   > : boost::asio::detail::thread_pool_bits
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
   typedef boost::asio::thread_pool::basic_executor_type<
       Allocator, Bits & ~blocking_mask> result_type;
 };
@@ -870,8 +743,8 @@ struct require_member<
     boost::asio::execution::blocking_t::always_t
   > : boost::asio::detail::thread_pool_bits
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = false);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = false;
   typedef boost::asio::thread_pool::basic_executor_type<Allocator,
       (Bits & ~blocking_mask) | blocking_always> result_type;
 };
@@ -882,8 +755,8 @@ struct require_member<
     boost::asio::execution::blocking_t::never_t
   > : boost::asio::detail::thread_pool_bits
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = false);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = false;
   typedef boost::asio::thread_pool::basic_executor_type<
       Allocator, Bits & ~blocking_mask> result_type;
 };
@@ -894,8 +767,8 @@ struct require_member<
     boost::asio::execution::relationship_t::fork_t
   >
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = false);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = false;
   typedef boost::asio::thread_pool::basic_executor_type<
       Allocator, Bits> result_type;
 };
@@ -906,8 +779,8 @@ struct require_member<
     boost::asio::execution::relationship_t::continuation_t
   >
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = false);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = false;
   typedef boost::asio::thread_pool::basic_executor_type<
       Allocator, Bits> result_type;
 };
@@ -918,8 +791,8 @@ struct require_member<
     boost::asio::execution::outstanding_work_t::tracked_t
   > : boost::asio::detail::thread_pool_bits
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = false);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = false;
   typedef boost::asio::thread_pool::basic_executor_type<
       Allocator, Bits | outstanding_work_tracked> result_type;
 };
@@ -930,8 +803,8 @@ struct require_member<
     boost::asio::execution::outstanding_work_t::untracked_t
   > : boost::asio::detail::thread_pool_bits
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = false);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = false;
   typedef boost::asio::thread_pool::basic_executor_type<
       Allocator, Bits & ~outstanding_work_tracked> result_type;
 };
@@ -942,8 +815,8 @@ struct require_member<
     boost::asio::execution::allocator_t<void>
   >
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = false);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = false;
   typedef boost::asio::thread_pool::basic_executor_type<
       std::allocator<void>, Bits> result_type;
 };
@@ -955,8 +828,8 @@ struct require_member<
     boost::asio::execution::allocator_t<OtherAllocator>
   >
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = false);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = false;
   typedef boost::asio::thread_pool::basic_executor_type<
       OtherAllocator, Bits> result_type;
 };
@@ -965,32 +838,6 @@ struct require_member<
 
 #if !defined(BOOST_ASIO_HAS_DEDUCED_QUERY_STATIC_CONSTEXPR_MEMBER_TRAIT)
 
-#if !defined(BOOST_ASIO_NO_DEPRECATED)
-
-template <typename Allocator, unsigned int Bits, typename Property>
-struct query_static_constexpr_member<
-    boost::asio::thread_pool::basic_executor_type<Allocator, Bits>,
-    Property,
-    typename boost::asio::enable_if<
-      boost::asio::is_convertible<
-        Property,
-        boost::asio::execution::bulk_guarantee_t
-      >::value
-    >::type
-  >
-{
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
-  typedef boost::asio::execution::bulk_guarantee_t::parallel_t result_type;
-
-  static BOOST_ASIO_CONSTEXPR result_type value() BOOST_ASIO_NOEXCEPT
-  {
-    return result_type();
-  }
-};
-
-#endif // !defined(BOOST_ASIO_NO_DEPRECATED)
-
 template <typename Allocator, unsigned int Bits, typename Property>
 struct query_static_constexpr_member<
     boost::asio::thread_pool::basic_executor_type<Allocator, Bits>,
@@ -1003,11 +850,11 @@ struct query_static_constexpr_member<
     >::type
   > : boost::asio::detail::thread_pool_bits
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
   typedef boost::asio::execution::outstanding_work_t result_type;
 
-  static BOOST_ASIO_CONSTEXPR result_type value() BOOST_ASIO_NOEXCEPT
+  static constexpr result_type value() noexcept
   {
     return (Bits & outstanding_work_tracked)
       ? execution::outstanding_work_t(execution::outstanding_work.tracked)
@@ -1027,11 +874,11 @@ struct query_static_constexpr_member<
     >::type
   >
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
   typedef boost::asio::execution::mapping_t::thread_t result_type;
 
-  static BOOST_ASIO_CONSTEXPR result_type value() BOOST_ASIO_NOEXCEPT
+  static constexpr result_type value() noexcept
   {
     return result_type();
   }
@@ -1053,8 +900,8 @@ struct query_member<
     >::type
   >
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
   typedef boost::asio::execution::blocking_t result_type;
 };
 
@@ -1070,8 +917,8 @@ struct query_member<
     >::type
   >
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
   typedef boost::asio::execution::relationship_t result_type;
 };
 
@@ -1081,8 +928,8 @@ struct query_member<
     boost::asio::execution::occupancy_t
   >
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
   typedef std::size_t result_type;
 };
 
@@ -1092,8 +939,8 @@ struct query_member<
     boost::asio::execution::context_t
   >
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
   typedef boost::asio::thread_pool& result_type;
 };
 
@@ -1103,8 +950,8 @@ struct query_member<
     boost::asio::execution::allocator_t<void>
   >
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
   typedef Allocator result_type;
 };
 
@@ -1114,8 +961,8 @@ struct query_member<
     boost::asio::execution::allocator_t<OtherAllocator>
   >
 {
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_valid = true);
-  BOOST_ASIO_STATIC_CONSTEXPR(bool, is_noexcept = true);
+  static constexpr bool is_valid = true;
+  static constexpr bool is_noexcept = true;
   typedef Allocator result_type;
 };
 
