@@ -14,6 +14,7 @@
 #include <boost/beast/core/buffer_traits.hpp>
 #include <boost/beast/core/buffers_prefix.hpp>
 #include <boost/beast/websocket/teardown.hpp>
+#include <boost/asio/append.hpp>
 #include <boost/asio/coroutine.hpp>
 #include <boost/assert.hpp>
 #include <boost/make_shared.hpp>
@@ -149,6 +150,7 @@ close() noexcept
         error_code ec;
         socket.close(ec);
     }
+#if !defined(BOOST_NO_EXCEPTIONS)
     try
     {
         timer.cancel();
@@ -156,6 +158,9 @@ close() noexcept
     catch(...)
     {
     }
+#else
+    timer.cancel();
+#endif
 }
 
 //------------------------------------------------------------------------------
@@ -293,7 +298,7 @@ public:
             // This can occur even if an existing async_read is in progress.
             // In this specific case, we will complete the async op with no error
             // in order to prevent assertions and/or internal corruption of the basic_stream
-            this->complete(false, error_code(), 0);
+            this->complete(false, error_code(), std::size_t{0});
         }
         else
         {
@@ -309,6 +314,26 @@ public:
     {
         BOOST_ASIO_CORO_REENTER(*this)
         {
+            // apply the timeout manually, otherwise
+            // behavior varies across platforms.
+            if(state().timer.expiry() <= now())
+            {
+                BOOST_ASIO_CORO_YIELD
+                {
+                    BOOST_ASIO_HANDLER_LOCATION((
+                        __FILE__, __LINE__,
+                        (isRead ? "basic_stream::async_read_some"
+                            : "basic_stream::async_write_some")));
+
+                    const auto ex = this->get_immediate_executor();
+                    net::dispatch(ex, net::append(std::move(*this), ec, 0));
+                }
+
+                impl_->close();
+                BOOST_BEAST_ASSIGN_EC(ec, beast::error::timeout);
+                goto upcall;
+            }
+
             // handle empty buffers
             if(detail::buffers_empty(b_))
             {
@@ -322,13 +347,6 @@ public:
 
                     async_perform(0, is_read{});
                 }
-                // apply the timeout manually, otherwise
-                // behavior varies across platforms.
-                if(state().timer.expiry() <= clock_type::now())
-                {
-                    impl_->close();
-                    BOOST_BEAST_ASSIGN_EC(ec, beast::error::timeout);
-                }
                 goto upcall;
             }
 
@@ -589,11 +607,20 @@ public:
 
 struct run_read_op
 {
+    basic_stream* self;
+
+    using executor_type = typename basic_stream::executor_type;
+
+    executor_type
+    get_executor() const noexcept
+    {
+        return self->get_executor();
+    }
+
     template<class ReadHandler, class Buffers>
     void
     operator()(
         ReadHandler&& h,
-        basic_stream* s,
         Buffers const& b)
     {
         // If you get an error on the following line it means
@@ -609,17 +636,26 @@ struct run_read_op
             true,
             Buffers,
             typename std::decay<ReadHandler>::type>(
-                std::forward<ReadHandler>(h), *s, b);
+                std::forward<ReadHandler>(h), *self, b);
     }
 };
 
 struct run_write_op
 {
+    basic_stream* self;
+
+    using executor_type = typename basic_stream::executor_type;
+
+    executor_type
+    get_executor() const noexcept
+    {
+        return self->get_executor();
+    }
+
     template<class WriteHandler, class Buffers>
     void
     operator()(
         WriteHandler&& h,
-        basic_stream* s,
         Buffers const& b)
     {
         // If you get an error on the following line it means
@@ -635,17 +671,26 @@ struct run_write_op
             false,
             Buffers,
             typename std::decay<WriteHandler>::type>(
-                std::forward<WriteHandler>(h), *s, b);
+                std::forward<WriteHandler>(h), *self, b);
     }
 };
 
 struct run_connect_op
 {
+    basic_stream* self;
+
+    using executor_type = typename basic_stream::executor_type;
+
+    executor_type
+    get_executor() const noexcept
+    {
+        return self->get_executor();
+    }
+
     template<class ConnectHandler>
     void
     operator()(
         ConnectHandler&& h,
-        basic_stream* s,
         endpoint_type const& ep)
     {
         // If you get an error on the following line it means
@@ -658,12 +703,22 @@ struct run_connect_op
             "ConnectHandler type requirements not met");
 
         connect_op<typename std::decay<ConnectHandler>::type>(
-            std::forward<ConnectHandler>(h), *s, ep);
+            std::forward<ConnectHandler>(h), *self, ep);
     }
 };
 
 struct run_connect_range_op
 {
+    basic_stream* self;
+
+    using executor_type = typename basic_stream::executor_type;
+
+    executor_type
+    get_executor() const noexcept
+    {
+        return self->get_executor();
+    }
+
     template<
         class RangeConnectHandler,
         class EndpointSequence,
@@ -671,7 +726,6 @@ struct run_connect_range_op
     void
     operator()(
         RangeConnectHandler&& h,
-        basic_stream* s,
         EndpointSequence const& eps,
         Condition const& cond)
     {
@@ -685,12 +739,22 @@ struct run_connect_range_op
             "RangeConnectHandler type requirements not met");
 
         connect_op<typename std::decay<RangeConnectHandler>::type>(
-            std::forward<RangeConnectHandler>(h), *s, eps, cond);
+            std::forward<RangeConnectHandler>(h), *self, eps, cond);
     }
 };
 
 struct run_connect_iter_op
 {
+    basic_stream* self;
+
+    using executor_type = typename basic_stream::executor_type;
+
+    executor_type
+    get_executor() const noexcept
+    {
+        return self->get_executor();
+    }
+
     template<
         class IteratorConnectHandler,
         class Iterator,
@@ -698,7 +762,6 @@ struct run_connect_iter_op
     void
     operator()(
         IteratorConnectHandler&& h,
-        basic_stream* s,
         Iterator begin, Iterator end,
         Condition const& cond)
     {
@@ -712,7 +775,7 @@ struct run_connect_iter_op
             "IteratorConnectHandler type requirements not met");
 
         connect_op<typename std::decay<IteratorConnectHandler>::type>(
-            std::forward<IteratorConnectHandler>(h), *s, begin, end, cond);
+            std::forward<IteratorConnectHandler>(h), *self, begin, end, cond);
     }
 };
 
@@ -878,9 +941,8 @@ async_connect(
     return net::async_initiate<
         ConnectHandler,
         void(error_code)>(
-            typename ops::run_connect_op{},
+            typename ops::run_connect_op{this},
             handler,
-            this,
             ep);
 }
 
@@ -888,8 +950,9 @@ template<class Protocol, class Executor, class RatePolicy>
 template<
     class EndpointSequence,
     BOOST_ASIO_COMPLETION_TOKEN_FOR(void(error_code, typename Protocol::endpoint)) RangeConnectHandler,
+    class,
     class>
-BOOST_ASIO_INITFN_RESULT_TYPE(RangeConnectHandler,void(error_code, typename Protocol::endpoint))
+BOOST_ASIO_INITFN_AUTO_RESULT_TYPE(RangeConnectHandler,void(error_code, typename Protocol::endpoint))
 basic_stream<Protocol, Executor, RatePolicy>::
 async_connect(
     EndpointSequence const& endpoints,
@@ -898,9 +961,8 @@ async_connect(
     return net::async_initiate<
         RangeConnectHandler,
         void(error_code, typename Protocol::endpoint)>(
-            typename ops::run_connect_range_op{},
+            typename ops::run_connect_range_op{this},
             handler,
-            this,
             endpoints,
             detail::any_endpoint{});
 }
@@ -910,8 +972,9 @@ template<
     class EndpointSequence,
     class ConnectCondition,
     BOOST_ASIO_COMPLETION_TOKEN_FOR(void(error_code, typename Protocol::endpoint)) RangeConnectHandler,
+    class,
     class>
-BOOST_ASIO_INITFN_RESULT_TYPE(RangeConnectHandler,void (error_code, typename Protocol::endpoint))
+BOOST_ASIO_INITFN_AUTO_RESULT_TYPE(RangeConnectHandler,void (error_code, typename Protocol::endpoint))
 basic_stream<Protocol, Executor, RatePolicy>::
 async_connect(
     EndpointSequence const& endpoints,
@@ -921,9 +984,8 @@ async_connect(
     return net::async_initiate<
         RangeConnectHandler,
         void(error_code, typename Protocol::endpoint)>(
-            typename ops::run_connect_range_op{},
+            typename ops::run_connect_range_op{this},
             handler,
-            this,
             endpoints,
             connect_condition);
 }
@@ -931,8 +993,9 @@ async_connect(
 template<class Protocol, class Executor, class RatePolicy>
 template<
     class Iterator,
-    BOOST_ASIO_COMPLETION_TOKEN_FOR(void(error_code, Iterator)) IteratorConnectHandler>
-BOOST_ASIO_INITFN_RESULT_TYPE(IteratorConnectHandler,void (error_code, Iterator))
+    BOOST_ASIO_COMPLETION_TOKEN_FOR(void(error_code, Iterator)) IteratorConnectHandler,
+    class>
+BOOST_ASIO_INITFN_AUTO_RESULT_TYPE(IteratorConnectHandler,void (error_code, Iterator))
 basic_stream<Protocol, Executor, RatePolicy>::
 async_connect(
     Iterator begin, Iterator end,
@@ -941,9 +1004,8 @@ async_connect(
     return net::async_initiate<
         IteratorConnectHandler,
         void(error_code, Iterator)>(
-            typename ops::run_connect_iter_op{},
+            typename ops::run_connect_iter_op{this},
             handler,
-            this,
             begin, end,
             detail::any_endpoint{});
 }
@@ -952,8 +1014,9 @@ template<class Protocol, class Executor, class RatePolicy>
 template<
     class Iterator,
     class ConnectCondition,
-    BOOST_ASIO_COMPLETION_TOKEN_FOR(void(error_code, Iterator)) IteratorConnectHandler>
-BOOST_ASIO_INITFN_RESULT_TYPE(IteratorConnectHandler,void (error_code, Iterator))
+    BOOST_ASIO_COMPLETION_TOKEN_FOR(void(error_code, Iterator)) IteratorConnectHandler,
+    class>
+BOOST_ASIO_INITFN_AUTO_RESULT_TYPE(IteratorConnectHandler,void (error_code, Iterator))
 basic_stream<Protocol, Executor, RatePolicy>::
 async_connect(
     Iterator begin, Iterator end,
@@ -963,9 +1026,8 @@ async_connect(
     return net::async_initiate<
         IteratorConnectHandler,
         void(error_code, Iterator)>(
-            typename ops::run_connect_iter_op{},
+            typename ops::run_connect_iter_op{this},
             handler,
-            this,
             begin, end,
             connect_condition);
 }
@@ -986,9 +1048,8 @@ async_read_some(
     return net::async_initiate<
         ReadHandler,
         void(error_code, std::size_t)>(
-            typename ops::run_read_op{},
+            typename ops::run_read_op{this},
             handler,
-            this,
             buffers);
 }
 
@@ -1006,9 +1067,8 @@ async_write_some(
     return net::async_initiate<
         WriteHandler,
         void(error_code, std::size_t)>(
-            typename ops::run_write_op{},
+            typename ops::run_write_op{this},
             handler,
-            this,
             buffers);
 }
 
