@@ -19,6 +19,11 @@
 #include <boost/move/iterator.hpp>
 #include <boost/move/make_unique.hpp>
 
+#if defined(BOOST_GCC) && (BOOST_GCC >= 40600)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-result"
+#endif
+
 #define BOOST_INTRUSIVE_HAS_MEMBER_FUNCTION_CALLABLE_WITH_FUNCNAME rebalance
 #define BOOST_INTRUSIVE_HAS_MEMBER_FUNCTION_CALLABLE_WITH_NS_BEG namespace boost { namespace container { namespace test {
 #define BOOST_INTRUSIVE_HAS_MEMBER_FUNCTION_CALLABLE_WITH_NS_END   }}}
@@ -26,6 +31,11 @@
 #define BOOST_INTRUSIVE_HAS_MEMBER_FUNCTION_CALLABLE_WITH_MAX 0
 #include <boost/intrusive/detail/has_member_function_callable_with.hpp>
 
+//#pragma GCC diagnostic ignored "-Wunused-result"
+#if defined(BOOST_GCC) && (BOOST_GCC >= 40600)
+#pragma GCC diagnostic pop
+#endif
+
 namespace boost{
 namespace container {
 namespace test{
@@ -112,7 +122,7 @@ int set_test_copyable(boost::container::dtl::true_type)
          return 1;
    }
    {
-      //Now, test copy constructor
+      //Now, test copy constructor with allocator
       MyBoostSet boostsetcopy(boostset, typename MyBoostSet::allocator_type());
       MyStdSet stdsetcopy(stdset);
 
@@ -151,8 +161,7 @@ int set_test ()
    {  //Set(beg, end, compare)
       IntType aux_vect[50];
       for(int i = 0; i < 50; ++i){
-         IntType move_me(i/2);
-         aux_vect[i] = boost::move(move_me);
+         aux_vect[i] = i/2;
       }
       int aux_vect2[50];
       for(int i = 0; i < 50; ++i){
@@ -160,8 +169,7 @@ int set_test ()
       }
       IntType aux_vect3[50];
       for(int i = 0; i < 50; ++i){
-         IntType move_me(i/2);
-         aux_vect3[i] = boost::move(move_me);
+         aux_vect3[i] = i/2;
       }
       ::boost::movelib::unique_ptr<MyBoostSet> const pboostset2 = ::boost::movelib::make_unique<MyBoostSet>
          (boost::make_move_iterator(&aux_vect[0]), boost::make_move_iterator(&aux_vect[0]+50), typename MyBoostSet::key_compare());
@@ -175,8 +183,7 @@ int set_test ()
    {  //Set(beg, end, alloc)
       IntType aux_vect[50];
       for(int i = 0; i < 50; ++i){
-         IntType move_me(i/2);
-         aux_vect[i] = boost::move(move_me);
+         aux_vect[i] = i/2;
       }
       int aux_vect2[50];
       for(int i = 0; i < 50; ++i){
@@ -184,8 +191,7 @@ int set_test ()
       }
       IntType aux_vect3[50];
       for(int i = 0; i < 50; ++i){
-         IntType move_me(i/2);
-         aux_vect3[i] = boost::move(move_me);
+         aux_vect3[i] = i/2;
       }
       ::boost::movelib::unique_ptr<MyBoostSet> const pboostset2 = ::boost::movelib::make_unique<MyBoostSet>
          (boost::make_move_iterator(&aux_vect[0]), boost::make_move_iterator(&aux_vect[0]+50), typename MyBoostSet::allocator_type());
@@ -199,8 +205,7 @@ int set_test ()
    {
       IntType aux_vect[50];
       for(int i = 0; i < 50; ++i){
-         IntType move_me(i/2);
-         aux_vect[i] = boost::move(move_me);
+         aux_vect[i] = i/2;
       }
       int aux_vect2[50];
       for(int i = 0; i < 50; ++i){
@@ -208,8 +213,7 @@ int set_test ()
       }
       IntType aux_vect3[50];
       for(int i = 0; i < 50; ++i){
-         IntType move_me(i/2);
-         aux_vect3[i] = boost::move(move_me);
+         aux_vect3[i] = i/2;
       }
 
       ::boost::movelib::unique_ptr<MyBoostSet> const pboostset2 = ::boost::movelib::make_unique<MyBoostSet>
@@ -239,8 +243,7 @@ int set_test ()
 
       //ordered range insertion
       for(int i = 0; i < 50; ++i){
-         IntType move_me(i);
-         aux_vect[i] = boost::move(move_me);
+         aux_vect[i] = i;
       }
 
       for(int i = 0; i < 50; ++i){
@@ -248,8 +251,7 @@ int set_test ()
       }
 
       for(int i = 0; i < 50; ++i){
-         IntType move_me(i);
-         aux_vect3[i] = boost::move(move_me);
+         aux_vect3[i] = i;
       }
 
       //some comparison operators
@@ -347,35 +349,60 @@ int set_test ()
       return 1;
    }
 
-   //Swapping test
-   MyBoostSet tmpboosteset2;
-   MyStdSet tmpstdset2;
-   MyBoostMultiSet tmpboostemultiset2;
-   MyStdMultiSet tmpstdmultiset2;
-   boostset.swap(tmpboosteset2);
-   stdset.swap(tmpstdset2);
-   boostmultiset.swap(tmpboostemultiset2);
-   stdmultiset.swap(tmpstdmultiset2);
-   boostset.swap(tmpboosteset2);
-   stdset.swap(tmpstdset2);
-   boostmultiset.swap(tmpboostemultiset2);
-   stdmultiset.swap(tmpstdmultiset2);
-   if(!CheckEqualContainers(boostset, stdset)){
-      std::cout << "Error in boostset.swap(tmpboosteset2)" << std::endl;
-      return 1;
-   }
-   if(!CheckEqualContainers(boostmultiset, stdmultiset)){
-      std::cout << "Error in boostmultiset.swap(tmpboostemultiset2)" << std::endl;
-      return 1;
+   {
+      //Swapping test
+      MyBoostSet tmpboosteset2;
+      MyStdSet tmpstdset2;
+      MyBoostMultiSet tmpboostemultiset2;
+      MyStdMultiSet tmpstdmultiset2;
+      boostset.swap(tmpboosteset2);
+      stdset.swap(tmpstdset2);
+      boostmultiset.swap(tmpboostemultiset2);
+      stdmultiset.swap(tmpstdmultiset2);
+      boostset.swap(tmpboosteset2);
+      stdset.swap(tmpstdset2);
+      boostmultiset.swap(tmpboostemultiset2);
+      stdmultiset.swap(tmpstdmultiset2);
+      if(!CheckEqualContainers(boostset, stdset)){
+         std::cout << "Error in boostset.swap(tmpboosteset2)" << std::endl;
+         return 1;
+      }
+      if(!CheckEqualContainers(boostmultiset, stdmultiset)){
+         std::cout << "Error in boostmultiset.swap(tmpboostemultiset2)" << std::endl;
+         return 1;
+      }
    }
 
+   //move constructor/assignment
+   {
+      MyBoostSet tmpboosteset2(boost::move(boostset));
+      if(!CheckEqualContainers(tmpboosteset2, stdset)){
+         std::cout << "Error in boostset move constructor " << std::endl;
+         return 1;
+      }
+      MyBoostMultiSet tmpboostemultiset2(boost::move(boostmultiset));
+      if(!CheckEqualContainers(tmpboostemultiset2, stdmultiset)){
+         std::cout << "Error in boostmultiset move constructor " << std::endl;
+         return 1;
+      }
+
+      boostset = boost::move(tmpboosteset2);
+      if(!CheckEqualContainers(boostset, stdset)){
+         std::cout << "Error in boostset move assignment" << std::endl;
+         return 1;
+      }
+      boostmultiset = boost::move(tmpboostemultiset2);
+      if(!CheckEqualContainers(boostmultiset, stdmultiset)){
+         std::cout << "Error in boostmultiset move assignment" << std::endl;
+         return 1;
+      }
+   }
    //Insertion from other container
    //Initialize values
    {
       IntType aux_vect[50];
       for(int i = 0; i < 50; ++i){
-         IntType move_me(-1);
-         aux_vect[i] = boost::move(move_me);
+         aux_vect[i] = -1;
       }
       int aux_vect2[50];
       for(int i = 0; i < 50; ++i){
@@ -383,8 +410,7 @@ int set_test ()
       }
       IntType aux_vect3[50];
       for(int i = 0; i < 50; ++i){
-         IntType move_me(-1);
-         aux_vect3[i] = boost::move(move_me);
+         aux_vect3[i] = -1;
       }
 
       boostset.insert(boost::make_move_iterator(&aux_vect[0]), boost::make_move_iterator(&aux_vect[0] + 50));
@@ -419,8 +445,7 @@ int set_test ()
    {
       IntType aux_vect[50];
       for(int i = 0; i < 50; ++i){
-         IntType move_me(-1);
-         aux_vect[i] = boost::move(move_me);
+         aux_vect[i] = -1;
       }
       int aux_vect2[50];
       for(int i = 0; i < 50; ++i){
@@ -428,20 +453,17 @@ int set_test ()
       }
       IntType aux_vect3[50];
       for(int i = 0; i < 50; ++i){
-         IntType move_me(-1);
-         aux_vect3[i] = boost::move(move_me);
+         aux_vect3[i] = -1;
       }
 
       IntType aux_vect4[50];
       for(int i = 0; i < 50; ++i){
-         IntType move_me(-1);
-         aux_vect4[i] = boost::move(move_me);
+         aux_vect4[i] = -1;
       }
 
       IntType aux_vect5[50];
       for(int i = 0; i < 50; ++i){
-         IntType move_me(-1);
-         aux_vect5[i] = boost::move(move_me);
+         aux_vect5[i] = -1;
       }
 
       boostset.insert(boost::make_move_iterator(&aux_vect[0]), boost::make_move_iterator(&aux_vect[0] + 50));
@@ -605,9 +627,7 @@ int set_test ()
       typename MyBoostSet::iterator bs_e = boostset.end();
       typename MyStdSet::iterator ss_b   = stdset.begin();
 
-      std::size_t i = 0;
       while(bs_b != bs_e){
-         ++i;
          typename MyBoostSet::iterator bs_i;
          typename MyStdSet::iterator ss_i;
          //find
@@ -728,16 +748,16 @@ int set_test ()
       stdmultiset.clear();
 
       {
-         IntType aux_vect[MaxElem];
+         IntType aux_vect[(std::size_t)MaxElem];
          for(int i = 0; i < MaxElem; ++i){
             aux_vect[i] = i;
          }
 
-         IntType aux_vect2[MaxElem];
+         IntType aux_vect2[(std::size_t)MaxElem];
          for(int i = 0; i < MaxElem; ++i){
             aux_vect2[i] = MaxElem/2+i;
          }
-         IntType aux_vect3[MaxElem];
+         IntType aux_vect3[(std::size_t)MaxElem];
          for(int i = 0; i < MaxElem; ++i){
             aux_vect3[i] = MaxElem*2/2+i;
          }
@@ -769,16 +789,16 @@ int set_test ()
       stdset.clear();
       stdmultiset.clear();
       {
-         IntType aux_vect[MaxElem];
+         IntType aux_vect[(std::size_t)MaxElem];
          for(int i = 0; i < MaxElem; ++i){
             aux_vect[i] = i;
          }
 
-         IntType aux_vect2[MaxElem];
+         IntType aux_vect2[(std::size_t)MaxElem];
          for(int i = 0; i < MaxElem; ++i){
             aux_vect2[i] = MaxElem/2+i;
          }
-         IntType aux_vect3[MaxElem];
+         IntType aux_vect3[(std::size_t)MaxElem];
          for(int i = 0; i < MaxElem; ++i){
             aux_vect3[i] = MaxElem*2/2+i;
          }
