@@ -18,8 +18,8 @@ struct foo
     operator long() const { return 0; }
 };
 
-typedef type<long> long_type;
-typedef type<foo> foo_type;
+using long_type = type<long>;
+using foo_type = type<foo>;
 
 int main()
 {
@@ -35,7 +35,7 @@ int main()
     (void)z;
 
 
-    BOOST_CONSTEXPR long value = boost::implicit_cast<long>(42);
+    constexpr long value = boost::implicit_cast<long>(42);
     BOOST_TEST(value == 42L);
 
     return boost::report_errors();
