@@ -2,8 +2,12 @@
 //
 // varray details
 //
-// Copyright (c) 2012-2020 Adam Wulkiewicz, Lodz, Poland.
 // Copyright (c) 2011-2013 Andrew Hundt.
+// Copyright (c) 2012-2023 Adam Wulkiewicz, Lodz, Poland.
+//
+// This file was modified by Oracle on 2020.
+// Modifications copyright (c) 2020, Oracle and/or its affiliates.
+// Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 //
 // Use, modification and distribution is subject to the Boost Software License,
 // Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
@@ -12,45 +16,19 @@
 #ifndef BOOST_GEOMETRY_INDEX_DETAIL_VARRAY_DETAIL_HPP
 #define BOOST_GEOMETRY_INDEX_DETAIL_VARRAY_DETAIL_HPP
 
+#include <algorithm>
 #include <cstddef>
 #include <cstring>
-#include <memory>
 #include <limits>
+#include <memory>
+#include <type_traits>
 
 #include <boost/config.hpp>
 
-#include <boost/mpl/if.hpp>
-#include <boost/mpl/and.hpp>
-#include <boost/mpl/or.hpp>
-#include <boost/mpl/int.hpp>
-
-#include <boost/type_traits/is_same.hpp>
-#include <boost/type_traits/remove_const.hpp>
-#include <boost/type_traits/remove_reference.hpp>
-#include <boost/type_traits/has_trivial_assign.hpp>
-#include <boost/type_traits/has_trivial_copy.hpp>
-#include <boost/type_traits/has_trivial_constructor.hpp>
-#include <boost/type_traits/has_trivial_destructor.hpp>
-#include <boost/type_traits/has_trivial_move_constructor.hpp>
-#include <boost/type_traits/has_trivial_move_assign.hpp>
-//#include <boost/type_traits/has_nothrow_constructor.hpp>
-//#include <boost/type_traits/has_nothrow_copy.hpp>
-//#include <boost/type_traits/has_nothrow_assign.hpp>
-//#include <boost/type_traits/has_nothrow_destructor.hpp>
-
-#if ! defined(BOOST_NO_CXX11_HDR_TYPE_TRAITS)
-#include <type_traits>
-#endif
-
-#include <boost/detail/no_exceptions_support.hpp>
-#include <boost/move/move.hpp>
 #include <boost/core/addressof.hpp>
+#include <boost/core/no_exceptions_support.hpp>
 #include <boost/iterator/iterator_traits.hpp>
 
-#if defined(BOOST_NO_CXX11_VARIADIC_TEMPLATES)
-#include <boost/move/detail/fwd_macros.hpp>
-#endif
-
 // TODO - move vectors iterators optimization to the other, optional file instead of checking defines?
 
 #if defined(BOOST_GEOMETRY_INDEX_DETAIL_VARRAY_ENABLE_VECTOR_OPTIMIZATION) && !defined(BOOST_NO_EXCEPTIONS)
@@ -61,60 +39,41 @@
 namespace boost { namespace geometry { namespace index { namespace detail { namespace varray_detail
 {
 
-#if ! defined(BOOST_NO_CXX11_HDR_TYPE_TRAITS)
-
-using std::is_trivially_copyable;
-
-#else
-
-template <typename T>
-struct is_trivially_copyable
-    : boost::integral_constant
-        <
-            bool,
-            boost::has_trivial_copy<T>::value
-         && boost::has_trivial_assign<T>::value
-         && boost::has_trivial_destructor<T>::value
-        >
-{};
-
-#endif
-
 
 template <typename I>
-struct are_elements_contiguous : boost::is_pointer<I>
+struct are_elements_contiguous : std::is_pointer<I>
 {};
-    
+
 // EXPERIMENTAL - not finished
 // Conditional setup - mark vector iterators defined in known implementations
 // as iterators pointing to contiguous ranges
 
 #if defined(BOOST_GEOMETRY_INDEX_DETAIL_VARRAY_ENABLE_VECTOR_OPTIMIZATION) && !defined(BOOST_NO_EXCEPTIONS)
-    
+
 template <typename Pointer>
 struct are_elements_contiguous<
     boost::container::container_detail::vector_const_iterator<Pointer>
-> : boost::true_type
+> : std::true_type
 {};
 
 template <typename Pointer>
 struct are_elements_contiguous<
     boost::container::container_detail::vector_iterator<Pointer>
-> : boost::true_type
+> : std::true_type
 {};
 
 #if defined(BOOST_DINKUMWARE_STDLIB)
-    
+
 template <typename T>
 struct are_elements_contiguous<
     std::_Vector_const_iterator<T>
-> : boost::true_type
+> : std::true_type
 {};
 
 template <typename T>
 struct are_elements_contiguous<
     std::_Vector_iterator<T>
-> : boost::true_type
+> : std::true_type
 {};
 
 #elif defined(BOOST_GNU_STDLIB)
@@ -122,7 +81,7 @@ struct are_elements_contiguous<
 template <typename P, typename T, typename A>
 struct are_elements_contiguous<
     __gnu_cxx::__normal_iterator<P, std::vector<T, A> >
-> : boost::true_type
+> : std::true_type
 {};
 
 #elif defined(_LIBCPP_VERSION)
@@ -131,55 +90,78 @@ struct are_elements_contiguous<
 //template <typename P>
 //struct are_elements_contiguous<
 //    __wrap_iter<P>
-//> : boost::true_type
+//> : std::true_type
 //{};
 
 #else // OTHER_STDLIB
 
 // TODO - add other iterators implementations
-    
+
 #endif // STDLIB
 
 #endif // BOOST_GEOMETRY_INDEX_DETAIL_VARRAY_ENABLE_VECTOR_OPTIMIZATION && !BOOST_NO_EXCEPTIONS
 
-// True if iterator values are the same and both iterators points to the ranges of contiguous elements
 
 template <typename I, typename O>
-struct are_corresponding :
-    ::boost::mpl::and_<
-        ::boost::is_same<
-            ::boost::remove_const<
-                typename ::boost::iterator_value<I>::type
-            >,
-            ::boost::remove_const<
-                typename ::boost::iterator_value<O>::type
-            >
-        >,
-        are_elements_contiguous<I>,
-        are_elements_contiguous<O>
-    >
+struct is_memop_safe_for_range
+    : std::integral_constant
+        <
+            bool,
+            std::is_same
+                <
+                    std::remove_const_t
+                        <
+                            typename ::boost::iterator_value<I>::type
+                        >,
+                    std::remove_const_t
+                        <
+                            typename ::boost::iterator_value<O>::type
+                        >
+                >::value
+            &&
+            are_elements_contiguous<I>::value
+            &&
+            are_elements_contiguous<O>::value
+            &&
+            std::is_trivially_copyable
+                <
+                    typename ::boost::iterator_value<O>::type
+                >::value
+        >
 {};
 
+
 template <typename I, typename V>
-struct is_corresponding_value :
-    ::boost::is_same<
-        ::boost::remove_const<
-            typename ::boost::iterator_value<I>::type
-        >,
-        ::boost::remove_const<V>
-    >
+struct is_memop_safe_for_value
+    : std::integral_constant
+        <
+            bool,
+            std::is_same
+                <
+                    std::remove_const_t
+                        <
+                            typename ::boost::iterator_value<I>::type
+                        >,
+                    std::remove_const_t<V>
+                >::value
+            &&
+            std::is_trivially_copyable
+                <
+                    V
+                >::value
+        >
 {};
 
 // destroy(I, I)
 
 template <typename I>
 void destroy_dispatch(I /*first*/, I /*last*/,
-                      boost::true_type const& /*has_trivial_destructor*/)
+                      std::true_type /*has_trivial_destructor*/)
 {}
 
 template <typename I>
 void destroy_dispatch(I first, I last,
-                      boost::false_type const& /*has_trivial_destructor*/)
+                      std::false_type /*has_trivial_destructor*/)
 {
     typedef typename boost::iterator_value<I>::type value_type;
     for ( ; first != last ; ++first )
@@ -190,19 +172,19 @@ template <typename I>
 void destroy(I first, I last)
 {
     typedef typename boost::iterator_value<I>::type value_type;
-    destroy_dispatch(first, last, boost::has_trivial_destructor<value_type>());
+    destroy_dispatch(first, last, std::is_trivially_destructible<value_type>());
 }
 
 // destroy(I)
 
 template <typename I>
 void destroy_dispatch(I /*pos*/,
-                      boost::true_type const& /*has_trivial_destructor*/)
+                      std::true_type /*has_trivial_destructor*/)
 {}
 
 template <typename I>
 void destroy_dispatch(I pos,
-                      boost::false_type const& /*has_trivial_destructor*/)
+                      std::false_type /*has_trivial_destructor*/)
 {
     typedef typename boost::iterator_value<I>::type value_type;
     pos->~value_type();
@@ -212,14 +194,14 @@ template <typename I>
 void destroy(I pos)
 {
     typedef typename boost::iterator_value<I>::type value_type;
-    destroy_dispatch(pos, boost::has_trivial_destructor<value_type>());
+    destroy_dispatch(pos, std::is_trivially_destructible<value_type>());
 }
 
 // copy(I, I, O)
 
 template <typename I, typename O>
 inline O copy_dispatch(I first, I last, O dst,
-                       boost::mpl::bool_<true> const& /*use_memmove*/)
+                       std::true_type /*use_memmove*/)
 {
     typedef typename boost::iterator_value<I>::type value_type;
     typename boost::iterator_difference<I>::type d = std::distance(first, last);
@@ -230,7 +212,7 @@ inline O copy_dispatch(I first, I last, O dst,
 
 template <typename I, typename O>
 inline O copy_dispatch(I first, I last, O dst,
-                       boost::mpl::bool_<false> const& /*use_memmove*/)
+                       std::false_type /*use_memmove*/)
 {
     return std::copy(first, last, dst);                                         // may throw
 }
@@ -238,16 +220,7 @@ inline O copy_dispatch(I first, I last, O dst,
 template <typename I, typename O>
 inline O copy(I first, I last, O dst)
 {
-    typedef typename
-    ::boost::mpl::and_<
-        are_corresponding<I, O>,
-        is_trivially_copyable<
-            typename ::boost::iterator_value<O>::type
-        >
-    >::type
-    use_memmove;
-    
-    return copy_dispatch(first, last, dst, use_memmove());                       // may throw
+    return copy_dispatch(first, last, dst, is_memop_safe_for_range<I, O>());    // may throw
 }
 
 // uninitialized_copy(I, I, O)
@@ -255,7 +228,7 @@ inline O copy(I first, I last, O dst)
 template <typename I, typename O>
 inline
 O uninitialized_copy_dispatch(I first, I last, O dst,
-                              boost::mpl::bool_<true> const& /*use_memcpy*/)
+                              std::true_type /*use_memcpy*/)
 {
     typedef typename boost::iterator_value<I>::type value_type;
     typename boost::iterator_difference<I>::type d = std::distance(first, last);
@@ -267,7 +240,7 @@ O uninitialized_copy_dispatch(I first, I last, O dst,
 template <typename I, typename F>
 inline
 F uninitialized_copy_dispatch(I first, I last, F dst,
-                              boost::mpl::bool_<false> const& /*use_memcpy*/)
+                              std::false_type /*use_memcpy*/)
 {
     return std::uninitialized_copy(first, last, dst);                                       // may throw
 }
@@ -276,16 +249,7 @@ template <typename I, typename F>
 inline
 F uninitialized_copy(I first, I last, F dst)
 {
-    typedef typename
-    ::boost::mpl::and_<
-        are_corresponding<I, F>,
-        is_trivially_copyable<
-            typename ::boost::iterator_value<F>::type
-        >
-    >::type
-    use_memcpy;
-
-    return uninitialized_copy_dispatch(first, last, dst, use_memcpy());          // may throw
+    return uninitialized_copy_dispatch(first, last, dst, is_memop_safe_for_range<I, F>());  // may throw
 }
 
 // uninitialized_move(I, I, O)
@@ -293,7 +257,7 @@ F uninitialized_copy(I first, I last, F dst)
 template <typename I, typename O>
 inline
 O uninitialized_move_dispatch(I first, I last, O dst,
-                              boost::mpl::bool_<true> const& /*use_memcpy*/)
+                              std::true_type /*use_memcpy*/)
 {
     typedef typename boost::iterator_value<I>::type value_type;
     typename boost::iterator_difference<I>::type d = std::distance(first, last);
@@ -305,7 +269,7 @@ O uninitialized_move_dispatch(I first, I last, O dst,
 template <typename I, typename O>
 inline
 O uninitialized_move_dispatch(I first, I last, O dst,
-                              boost::mpl::bool_<false> const& /*use_memcpy*/)
+                              std::false_type /*use_memcpy*/)
 {
     //return boost::uninitialized_move(first, last, dst);                         // may throw
 
@@ -315,11 +279,11 @@ O uninitialized_move_dispatch(I first, I last, O dst,
     {
         typedef typename std::iterator_traits<O>::value_type value_type;
         for (; first != last; ++first, ++o )
-            new (boost::addressof(*o)) value_type(boost::move(*first));
+            new (boost::addressof(*o)) value_type(std::move(*first));
     }
     BOOST_CATCH(...)
     {
-        destroy(dst, o);
+        varray_detail::destroy(dst, o);
         BOOST_RETHROW;
     }
     BOOST_CATCH_END
@@ -331,16 +295,7 @@ template <typename I, typename O>
 inline
 O uninitialized_move(I first, I last, O dst)
 {
-    typedef typename
-    ::boost::mpl::and_<
-        are_corresponding<I, O>,
-        is_trivially_copyable<
-            typename ::boost::iterator_value<O>::type
-        >
-    >::type
-    use_memcpy;
-
-    return uninitialized_move_dispatch(first, last, dst, use_memcpy());         // may throw
+    return uninitialized_move_dispatch(first, last, dst, is_memop_safe_for_range<I, O>()); // may throw
 }
 
 // TODO - move uses memmove - implement 2nd version using memcpy?
@@ -350,7 +305,7 @@ O uninitialized_move(I first, I last, O dst)
 template <typename I, typename O>
 inline
 O move_dispatch(I first, I last, O dst,
-                boost::mpl::bool_<true> const& /*use_memmove*/)
+                std::true_type /*use_memmove*/)
 {
     typedef typename boost::iterator_value<I>::type value_type;
     typename boost::iterator_difference<I>::type d = std::distance(first, last);
@@ -362,25 +317,16 @@ O move_dispatch(I first, I last, O dst,
 template <typename I, typename O>
 inline
 O move_dispatch(I first, I last, O dst,
-                boost::mpl::bool_<false> const& /*use_memmove*/)
+                std::false_type /*use_memmove*/)
 {
-    return boost::move(first, last, dst);                                         // may throw
+    return std::move(first, last, dst);                                           // may throw
 }
 
 template <typename I, typename O>
 inline
 O move(I first, I last, O dst)
 {
-    typedef typename
-    ::boost::mpl::and_<
-        are_corresponding<I, O>,
-        is_trivially_copyable<
-            typename ::boost::iterator_value<O>::type
-        >
-    >::type
-    use_memmove;
-
-    return move_dispatch(first, last, dst, use_memmove());                      // may throw
+    return move_dispatch(first, last, dst, is_memop_safe_for_range<I, O>());      // may throw
 }
 
 // move_backward(BDI, BDI, BDO)
@@ -388,7 +334,7 @@ O move(I first, I last, O dst)
 template <typename BDI, typename BDO>
 inline
 BDO move_backward_dispatch(BDI first, BDI last, BDO dst,
-                           boost::mpl::bool_<true> const& /*use_memmove*/)
+                           std::true_type /*use_memmove*/)
 {
     typedef typename boost::iterator_value<BDI>::type value_type;
     typename boost::iterator_difference<BDI>::type d = std::distance(first, last);
@@ -401,60 +347,43 @@ BDO move_backward_dispatch(BDI first, BDI last, BDO dst,
 template <typename BDI, typename BDO>
 inline
 BDO move_backward_dispatch(BDI first, BDI last, BDO dst,
-                           boost::mpl::bool_<false> const& /*use_memmove*/)
+                           std::false_type /*use_memmove*/)
 {
-    return boost::move_backward(first, last, dst);                                // may throw
+    return std::move_backward(first, last, dst);                                  // may throw
 }
 
 template <typename BDI, typename BDO>
 inline
 BDO move_backward(BDI first, BDI last, BDO dst)
 {
-    typedef typename
-    ::boost::mpl::and_<
-        are_corresponding<BDI, BDO>,
-        is_trivially_copyable<
-            typename ::boost::iterator_value<BDO>::type
-        >
-    >::type
-    use_memmove;
-
-    return move_backward_dispatch(first, last, dst, use_memmove());             // may throw
+    return move_backward_dispatch(first, last, dst, is_memop_safe_for_range<BDI, BDO>()); // may throw
 }
 
-template <typename T>
-struct has_nothrow_move : public
-    ::boost::mpl::or_<
-        boost::mpl::bool_<
-            ::boost::has_nothrow_move<
-                typename ::boost::remove_const<T>::type
-            >::value
-        >,
-        boost::mpl::bool_<
-            ::boost::has_nothrow_move<T>::value
-        >
-    >
-{};
-
 // uninitialized_move_if_noexcept(I, I, O)
 
 template <typename I, typename O>
 inline
-O uninitialized_move_if_noexcept_dispatch(I first, I last, O dst, boost::mpl::bool_<true> const& /*use_move*/)
-{ return varray_detail::uninitialized_move(first, last, dst); }
+O uninitialized_move_if_noexcept_dispatch(I first, I last, O dst,
+                                          std::true_type /*use_move*/)
+{
+    return varray_detail::uninitialized_move(first, last, dst);
+}
 
 template <typename I, typename O>
 inline
-O uninitialized_move_if_noexcept_dispatch(I first, I last, O dst, boost::mpl::bool_<false> const& /*use_move*/)
-{ return varray_detail::uninitialized_copy(first, last, dst); }
+O uninitialized_move_if_noexcept_dispatch(I first, I last, O dst,
+                                          std::false_type const& /*use_move*/)
+{
+    return varray_detail::uninitialized_copy(first, last, dst);
+}
 
 template <typename I, typename O>
 inline
 O uninitialized_move_if_noexcept(I first, I last, O dst)
 {
-    typedef typename has_nothrow_move<
+    typedef std::is_nothrow_move_constructible<
         typename ::boost::iterator_value<O>::type
-    >::type use_move;
+    > use_move;
 
     return uninitialized_move_if_noexcept_dispatch(first, last, dst, use_move());         // may throw
 }
@@ -463,21 +392,27 @@ O uninitialized_move_if_noexcept(I first, I last, O dst)
 
 template <typename I, typename O>
 inline
-O move_if_noexcept_dispatch(I first, I last, O dst, boost::mpl::bool_<true> const& /*use_move*/)
-{ return move(first, last, dst); }
+O move_if_noexcept_dispatch(I first, I last, O dst,
+                            std::true_type /*use_move*/)
+{
+    return varray_detail::move(first, last, dst);
+}
 
 template <typename I, typename O>
 inline
-O move_if_noexcept_dispatch(I first, I last, O dst, boost::mpl::bool_<false> const& /*use_move*/)
-{ return copy(first, last, dst); }
+O move_if_noexcept_dispatch(I first, I last, O dst,
+                            std::false_type /*use_move*/)
+{
+    return varray_detail::copy(first, last, dst);
+}
 
 template <typename I, typename O>
 inline
 O move_if_noexcept(I first, I last, O dst)
 {
-    typedef typename has_nothrow_move<
+    typedef std::is_nothrow_move_constructible<
         typename ::boost::iterator_value<O>::type
-    >::type use_move;
+    > use_move;
 
     return move_if_noexcept_dispatch(first, last, dst, use_move());         // may throw
 }
@@ -487,15 +422,15 @@ O move_if_noexcept(I first, I last, O dst)
 template <typename I>
 inline
 void uninitialized_fill_dispatch(I /*first*/, I /*last*/,
-                                 boost::true_type const& /*has_trivial_constructor*/,
-                                 boost::true_type const& /*disable_trivial_init*/)
+                                 std::true_type const& /*has_trivial_constructor*/,
+                                 std::true_type const& /*disable_trivial_init*/)
 {}
 
 template <typename I>
 inline
 void uninitialized_fill_dispatch(I first, I last,
-                                 boost::true_type const& /*has_trivial_constructor*/,
-                                 boost::false_type const& /*disable_trivial_init*/)
+                                 std::true_type const& /*has_trivial_constructor*/,
+                                 std::false_type const& /*disable_trivial_init*/)
 {
     typedef typename boost::iterator_value<I>::type value_type;
     for ( ; first != last ; ++first )
@@ -505,7 +440,7 @@ void uninitialized_fill_dispatch(I first, I last,
 template <typename I, typename DisableTrivialInit>
 inline
 void uninitialized_fill_dispatch(I first, I last,
-                                 boost::false_type const& /*has_trivial_constructor*/,
+                                 std::false_type const& /*has_trivial_constructor*/,
                                  DisableTrivialInit const& /*not_used*/)
 {
     typedef typename boost::iterator_value<I>::type value_type;
@@ -518,7 +453,7 @@ void uninitialized_fill_dispatch(I first, I last,
     }
     BOOST_CATCH(...)
     {
-        destroy(first, it);
+        varray_detail::destroy(first, it);
         BOOST_RETHROW;
     }
     BOOST_CATCH_END
@@ -529,22 +464,22 @@ inline
 void uninitialized_fill(I first, I last, DisableTrivialInit const& disable_trivial_init)
 {
     typedef typename boost::iterator_value<I>::type value_type;
-    uninitialized_fill_dispatch(first, last, boost::has_trivial_constructor<value_type>(), disable_trivial_init);     // may throw
+    uninitialized_fill_dispatch(first, last, std::is_trivially_constructible<value_type>(), disable_trivial_init);     // may throw
 }
 
 // construct(I)
 
 template <typename I>
 inline
-void construct_dispatch(boost::mpl::bool_<true> const& /*dont_init*/, I /*pos*/)
+void construct_dispatch(std::true_type /*dont_init*/, I /*pos*/)
 {}
 
 template <typename I>
 inline
-void construct_dispatch(boost::mpl::bool_<false> const& /*dont_init*/, I pos)
+void construct_dispatch(std::false_type /*dont_init*/, I pos)
 {
     typedef typename ::boost::iterator_value<I>::type value_type;
-    new (static_cast<void*>(::boost::addressof(*pos))) value_type();                      // may throw
+    new (static_cast<void*>(boost::addressof(*pos))) value_type();                      // may throw
 }
 
 template <typename DisableTrivialInit, typename I>
@@ -552,10 +487,13 @@ inline
 void construct(DisableTrivialInit const&, I pos)
 {
     typedef typename ::boost::iterator_value<I>::type value_type;
-    typedef typename ::boost::mpl::and_<
-        boost::has_trivial_constructor<value_type>,
-        DisableTrivialInit
-    >::type dont_init;
+    typedef std::integral_constant
+        <
+            bool,
+            std::is_trivially_constructible<value_type>::value
+            &&
+            DisableTrivialInit::value
+        > dont_init;
 
     construct_dispatch(dont_init(), pos);                                                // may throw
 }
@@ -565,7 +503,7 @@ void construct(DisableTrivialInit const&, I pos)
 template <typename I, typename V>
 inline
 void construct_copy_dispatch(I pos, V const& v,
-                             boost::mpl::bool_<true> const& /*use_memcpy*/)
+                             std::true_type /*use_memcpy*/)
 {
     ::memcpy(boost::addressof(*pos), boost::addressof(v), sizeof(V));
 }
@@ -573,7 +511,7 @@ void construct_copy_dispatch(I pos, V const& v,
 template <typename I, typename P>
 inline
 void construct_copy_dispatch(I pos, P const& p,
-                             boost::mpl::bool_<false> const& /*use_memcpy*/)
+                             std::false_type const& /*use_memcpy*/)
 {
     typedef typename boost::iterator_value<I>::type V;
     new (static_cast<void*>(boost::addressof(*pos))) V(p);                      // may throw
@@ -584,14 +522,7 @@ inline
 void construct(DisableTrivialInit const&,
                I pos, P const& p)
 {
-    typedef typename
-    ::boost::mpl::and_<
-        is_corresponding_value<I, P>,
-        is_trivially_copyable<P>
-    >::type
-    use_memcpy;
-
-    construct_copy_dispatch(pos, p, use_memcpy());                              // may throw
+    construct_copy_dispatch(pos, p, is_memop_safe_for_value<I, P>());           // may throw
 }
 
 // Needed by push_back(V &&)
@@ -599,81 +530,45 @@ void construct(DisableTrivialInit const&,
 template <typename I, typename V>
 inline
 void construct_move_dispatch(I pos, V const& v,
-                             boost::mpl::bool_<true> const& /*use_memcpy*/)
+                             std::true_type const& /*use_memcpy*/)
 {
     ::memcpy(boost::addressof(*pos), boost::addressof(v), sizeof(V));
 }
 
 template <typename I, typename P>
 inline
-void construct_move_dispatch(I pos, BOOST_RV_REF(P) p,
-                             boost::mpl::bool_<false> const& /*use_memcpy*/)
+void construct_move_dispatch(I pos, P&& p,
+                             std::false_type const& /*use_memcpy*/)
 {
     typedef typename boost::iterator_value<I>::type V;
-    new (static_cast<void*>(boost::addressof(*pos))) V(::boost::move(p));       // may throw
+    new (static_cast<void*>(boost::addressof(*pos))) V(std::move(p));       // may throw
 }
 
 template <typename DisableTrivialInit, typename I, typename P>
 inline
-void construct(DisableTrivialInit const&, I pos, BOOST_RV_REF(P) p)
+void construct(DisableTrivialInit const&, I pos, P&& p)
 {
-    typedef typename
-    ::boost::mpl::and_<
-        is_corresponding_value<I, P>,
-        ::boost::has_trivial_move_constructor<P>
-    >::type
-    use_memcpy;
-
-    construct_move_dispatch(pos, ::boost::move(p), use_memcpy());               // may throw
+    construct_move_dispatch(pos, std::move(p), is_memop_safe_for_value<I, P>()); // may throw
 }
 
 // Needed by emplace_back() and emplace()
 
-#if !defined(BOOST_CONTAINER_VARRAY_DISABLE_EMPLACE)
-#if !defined(BOOST_NO_CXX11_VARIADIC_TEMPLATES)
-
 template <typename DisableTrivialInit, typename I, class ...Args>
 inline
 void construct(DisableTrivialInit const&,
                I pos,
-               BOOST_FWD_REF(Args) ...args)
+               Args&& ...args)
 {
     typedef typename boost::iterator_value<I>::type V;
-    new (static_cast<void*>(boost::addressof(*pos))) V(::boost::forward<Args>(args)...);    // may throw
+    new (static_cast<void*>(boost::addressof(*pos))) V(std::forward<Args>(args)...);    // may throw
 }
 
-#else // !BOOST_NO_CXX11_VARIADIC_TEMPLATES
-
-// BOOST_NO_CXX11_RVALUE_REFERENCES -> P0 const& p0
-// !BOOST_NO_CXX11_RVALUE_REFERENCES -> P0 && p0
-// which means that version with one parameter may take V const& v
-
-#define BOOST_GEOMETRY_INDEX_DETAIL_VARRAY_DETAIL_CONSTRUCT(N)                                      \
-template <typename DisableTrivialInit, typename I, typename P BOOST_MOVE_I##N BOOST_MOVE_CLASS##N > \
-inline                                                                                              \
-void construct(DisableTrivialInit const&,                                                           \
-               I pos,                                                                               \
-               BOOST_FWD_REF(P) p                                                                   \
-               BOOST_MOVE_I##N BOOST_MOVE_UREF##N)                                                  \
-{                                                                                                   \
-    typedef typename boost::iterator_value<I>::type V;                                              \
-    new                                                                                             \
-    (static_cast<void*>(boost::addressof(*pos)))                                                    \
-    V(boost::forward<P>(p) BOOST_MOVE_I##N BOOST_MOVE_FWD##N);                    /*may throw*/    \
-}                                                                                                   \
-
-BOOST_MOVE_ITERATE_1TO9(BOOST_GEOMETRY_INDEX_DETAIL_VARRAY_DETAIL_CONSTRUCT)
-#undef BOOST_GEOMETRY_INDEX_DETAIL_VARRAY_DETAIL_CONSTRUCT
-
-#endif // !BOOST_NO_CXX11_VARIADIC_TEMPLATES
-#endif // !BOOST_CONTAINER_VARRAY_DISABLE_EMPLACE
-
 // assign(I, V)
 
 template <typename I, typename V>
 inline
 void assign_copy_dispatch(I pos, V const& v,
-                          boost::mpl::bool_<true> const& /*use_memcpy*/)
+                          std::true_type /*use_memcpy*/)
 {
 // TODO - use memmove here?
     ::memcpy(boost::addressof(*pos), boost::addressof(v), sizeof(V));
@@ -682,7 +577,7 @@ void assign_copy_dispatch(I pos, V const& v,
 template <typename I, typename V>
 inline
 void assign_copy_dispatch(I pos, V const& v,
-                          boost::mpl::bool_<false> const& /*use_memcpy*/)
+                          std::false_type /*use_memcpy*/)
 {
     *pos = v;                                                                   // may throw
 }
@@ -691,20 +586,13 @@ template <typename I, typename V>
 inline
 void assign(I pos, V const& v)
 {
-    typedef typename
-    ::boost::mpl::and_<
-        is_corresponding_value<I, V>,
-        is_trivially_copyable<V>
-    >::type
-    use_memcpy;
-
-    assign_copy_dispatch(pos, v, use_memcpy());                                   // may throw
+    assign_copy_dispatch(pos, v, is_memop_safe_for_value<I, V>());              // may throw
 }
 
 template <typename I, typename V>
 inline
 void assign_move_dispatch(I pos, V const& v,
-                          boost::mpl::bool_<true> const& /*use_memcpy*/)
+                          std::true_type /*use_memcpy*/)
 {
 // TODO - use memmove here?
     ::memcpy(boost::addressof(*pos), boost::addressof(v), sizeof(V));
@@ -712,24 +600,17 @@ void assign_move_dispatch(I pos, V const& v,
 
 template <typename I, typename V>
 inline
-void assign_move_dispatch(I pos, BOOST_RV_REF(V) v,
-                          boost::mpl::bool_<false> const& /*use_memcpy*/)
+void assign_move_dispatch(I pos, V&& v,
+                          std::false_type /*use_memcpy*/)
 {
-    *pos = boost::move(v);                                                        // may throw
+    *pos = std::move(v);                                                          // may throw
 }
 
 template <typename I, typename V>
 inline
-void assign(I pos, BOOST_RV_REF(V) v)
+void assign(I pos, V&& v)
 {
-    typedef typename
-    ::boost::mpl::and_<
-        is_corresponding_value<I, V>,
-        ::boost::has_trivial_move_assign<V>
-    >::type
-    use_memcpy;
-
-    assign_move_dispatch(pos, ::boost::move(v), use_memcpy());
+    assign_move_dispatch(pos, std::move(v), is_memop_safe_for_value<I, V>());
 }
 
 // uninitialized_copy_s
@@ -753,7 +634,7 @@ inline std::size_t uninitialized_copy_s(I first, I last, F dest, std::size_t max
     }
     BOOST_CATCH(...)
     {
-        destroy(dest, it);
+        varray_detail::destroy(dest, it);
         BOOST_RETHROW;
     }
     BOOST_CATCH_END
@@ -772,7 +653,7 @@ public:
     ~scoped_destructor()
     {
         if(m_ptr)
-            destroy(m_ptr);
+            varray_detail::destroy(m_ptr);
     }
 
     void release() { m_ptr = 0; }
