@@ -26,8 +26,10 @@
 
 #include <boost/geometry/geometries/point_xy.hpp>
 #include <boost/geometry/geometries/register/linestring.hpp>
+#include <boost/geometry/algorithms/detail/has_self_intersections.hpp>
 
 #include <boost/geometry/util/condition.hpp>
+#include <boost/geometry/util/constexpr.hpp>
 #include <boost/geometry/util/rational.hpp>
 
 #include "test_intersection.hpp"
@@ -50,11 +52,6 @@ BOOST_GEOMETRY_REGISTER_LINESTRING_TEMPLATED(std::vector)
     (test_one<Polygon, Polygon, Polygon>) \
     ( #caseid "_rev", caseid[1], caseid[0], clips, points, area)
 
-#define TEST_INTERSECTION_IGNORE(caseid, clips, points, area) \
-    { ut_settings ignore_validity; ignore_validity.set_test_validity(false); \
-    (test_one<Polygon, Polygon, Polygon>) \
-    ( #caseid, caseid[0], caseid[1], clips, points, area, ignore_validity); }
-
 #define TEST_INTERSECTION_WITH(caseid, index1, index2, \
      clips, points, area, settings) \
     (test_one<Polygon, Polygon, Polygon>) \
@@ -150,13 +147,9 @@ void test_areal()
         simplex_normal[0], simplex_normal[1],
         1, 7, 5.47363293);
 
-#if ! defined(BOOST_GEOMETRY_USE_RESCALING) || defined(BOOST_GEOMETRY_TEST_FAILURES)
-    // Fails if rescaling is used in combination with get_clusters, because a cluster is generated
-    // and use as the start of the traversal
     test_one<Polygon, Polygon, Polygon>("distance_zero",
         distance_zero[0], distance_zero[1],
         1, 0, 0.29516139);
-#endif
 
     test_one<Polygon, Polygon, Polygon>("equal_holes_disjoint",
         equal_holes_disjoint[0], equal_holes_disjoint[1],
@@ -181,14 +174,9 @@ void test_areal()
         pie_2_3_23_0[0], pie_2_3_23_0[1],
         1, 4, 163292.679042133, ut_settings(0.1));
 
-#if defined(BOOST_GEOMETRY_USE_RESCALING) || defined(BOOST_GEOMETRY_TEST_FAILURES)
     TEST_INTERSECTION(isovist, 1, 19, expectation_limits(88.19202, 88.19206));
-#else
-    // Reported as invalid without rescaling and get_clusters
-    TEST_INTERSECTION_IGNORE(isovist, 1, 19, expectation_limits(88.19202, 88.19206));
-#endif
 
-    TEST_INTERSECTION_IGNORE(geos_1, 1, -1, expectation_limits(3454, 3462));
+    TEST_INTERSECTION(geos_1, 1, -1, expectation_limits(3454, 3462));
 
     // Can, in some cases, create small slivers
     // In some cases: 1.430511474609375e-05 (clang/gcc on Xubuntu using b2)
@@ -290,25 +278,23 @@ void test_areal()
     {
         // Not yet valid when rescaling is turned off
         ut_settings settings;
-        settings.set_test_validity(BG_IF_RESCALED(true, false));
+        settings.set_test_validity(false);
         test_one<Polygon, Polygon, Polygon>("ticket_9563", ticket_9563[0], ticket_9563[1],
                     1, 8, 129.90381, settings);
     }
 
-#if ! defined(BOOST_GEOMETRY_USE_RESCALING) || defined(BOOST_GEOMETRY_TEST_FAILURES)
-    // With rescaling the output is empty
     TEST_INTERSECTION(issue_548, 1, -1, expectation_limits(1958821942, 1958824416));
-#endif
 
     TEST_INTERSECTION(issue_566_a, 1, -1, 70.7107);
     TEST_INTERSECTION(issue_566_b, 1, -1, 70.7107);
 
     TEST_INTERSECTION(issue_838, 1, -1, (expectation_limits{0.6582, 0.6650}));
 
-#if ! defined(BOOST_GEOMETRY_USE_RESCALING) || defined(BOOST_GEOMETRY_TEST_FAILURES)
-    // With rescaling the output is wrong
     TEST_INTERSECTION(issue_861, 1, -1, 1.4715007684573677693e-10);
-#endif
+
+    TEST_INTERSECTION(issue_893, 1, -1, 473001.5082956461);
+
+    TEST_INTERSECTION(issue_1229, 0, -1, 0);
 
     test_one<Polygon, Polygon, Polygon>("buffer_mp1", buffer_mp1[0], buffer_mp1[1],
                 1, 31, 2.271707796);
@@ -414,15 +400,9 @@ void test_areal()
 
     TEST_INTERSECTION(mysql_23023665_6, 2, 0, 11.812440191387557);
 
-    // Formation of an interior ring is optional
-    test_one<Polygon, Polygon, Polygon>("mysql_23023665_10",
-        mysql_23023665_10[0], mysql_23023665_10[1],
-        1, optional(), -1, 54.701340543162523);
-
-    // Formation of an interior ring is optional
-    test_one<Polygon, Polygon, Polygon>("mysql_23023665_11",
-        mysql_23023665_11[0], mysql_23023665_11[1],
-        1, optional(), -1, 35.933385462482065);
+    // Formation of an interior ring is optional for these cases
+    TEST_INTERSECTION(mysql_23023665_10, optional(), 1, 54.701340543162523);
+    TEST_INTERSECTION(mysql_23023665_11, optional(), 1, 35.933385462482065);
 
 //    test_one<Polygon, Polygon, Polygon>(
 //        "polygon_pseudo_line",
@@ -686,9 +666,11 @@ void test_all()
     std::string clip = "box(2 2,8 8)";
 
     test_areal_linear<polygon, linestring>();
+#if ! defined(BOOST_GEOMETRY_TEST_ONLY_ONE_ORDER)
+    test_areal_linear<polygon_ccw, linestring>();
+#endif
 #if ! defined(BOOST_GEOMETRY_TEST_ONLY_ONE_TYPE)
     test_areal_linear<polygon_open, linestring>();
-    test_areal_linear<polygon_ccw, linestring>();
     test_areal_linear<polygon_ccw_open, linestring>();
 #endif
 
@@ -697,8 +679,10 @@ void test_all()
     // Test polygons clockwise and counter clockwise
     test_areal<polygon>();
 
-#if ! defined(BOOST_GEOMETRY_TEST_ONLY_ONE_TYPE)
+#if ! defined(BOOST_GEOMETRY_TEST_ONLY_ONE_ORDER)
     test_areal<polygon_ccw>();
+#endif
+#if ! defined(BOOST_GEOMETRY_TEST_ONLY_ONE_TYPE)
     test_areal<polygon_open>();
     test_areal<polygon_ccw_open>();
 #endif
@@ -752,7 +736,7 @@ void test_all()
 
     // polygons outputing points
     //test_one<P, polygon, polygon>("ppp1", simplex_normal[0], simplex_normal[1], 1, 7, 5.47363293);
-    
+
     test_point_output<P>();
 
     /*
@@ -826,33 +810,6 @@ void test_rational()
         1, 7, 5.47363293);
 }
 
-template <typename CoordinateType>
-void test_ticket_10868(std::string const& wkt_out)
-{
-    typedef bg::model::point<CoordinateType, 2, bg::cs::cartesian> point_type;
-    typedef bg::model::polygon
-        <
-            point_type, /*ClockWise*/false, /*Closed*/false
-        > polygon_type;
-    typedef bg::model::multi_polygon<polygon_type> multipolygon_type;
-
-    polygon_type polygon1;
-    bg::read_wkt(ticket_10868[0], polygon1);
-    polygon_type polygon2;
-    bg::read_wkt(ticket_10868[1], polygon2);
-
-    multipolygon_type multipolygon_out;
-    bg::intersection(polygon1, polygon2, multipolygon_out);
-    std::stringstream stream;
-    stream << bg::wkt(multipolygon_out);
-
-    BOOST_CHECK_EQUAL(stream.str(), wkt_out);
-
-    test_one<polygon_type, polygon_type, polygon_type>("ticket_10868",
-        ticket_10868[0], ticket_10868[1],
-        1, 7, 20266195244586.0);
-}
-
 int test_main(int, char* [])
 {
     BoostGeometryWriteTestConfiguration();
@@ -865,32 +822,8 @@ int test_main(int, char* [])
     // test_exception<bg::model::d2::point_xy<double> >();
 
     test_pointer_version();
-#if ! defined(BOOST_GEOMETRY_RESCALE_TO_ROBUST)
     test_rational<bg::model::d2::point_xy<boost::rational<int> > >();
 #endif
 
-#if defined(BOOST_GEOMETRY_TEST_FAILURES)
-    // ticket #10868 still fails for 32-bit integers
-    test_ticket_10868<int32_t>("MULTIPOLYGON(((33520458 6878575,33480192 14931538,31446819 18947953,30772384 19615678,30101303 19612322,30114725 16928001,33520458 6878575)))");
-
-#if !defined(BOOST_NO_INT64_T) || defined(BOOST_HAS_MS_INT64)
-    test_ticket_10868<int64_t>("MULTIPOLYGON(((33520458 6878575,33480192 14931538,31446819 18947953,30772384 19615678,30101303 19612322,30114725 16928001,33520458 6878575)))");
-#endif
-
-    if (BOOST_GEOMETRY_CONDITION(sizeof(long) * CHAR_BIT >= 64))
-    {
-        test_ticket_10868<long>("MULTIPOLYGON(((33520458 6878575,33480192 14931538,31446819 18947953,30772384 19615678,30101303 19612322,30114725 16928001,33520458 6878575)))");
-    }
-
-    test_ticket_10868<long long>("MULTIPOLYGON(((33520458 6878575,33480192 14931538,31446819 18947953,30772384 19615678,30101303 19612322,30114725 16928001,33520458 6878575)))");
-#endif
-#endif
-
-#if defined(BOOST_GEOMETRY_TEST_FAILURES)
-    // llb_touch generates a polygon with 1 point and is therefore invalid everywhere
-    // TODO: this should be easy to fix
-    BoostGeometryWriteExpectedFailures(6, 2, 7, 1);
-#endif
-
     return 0;
 }
