@@ -11,7 +11,6 @@
 #define BOOST_JSON_ERROR_HPP
 
 #include <boost/json/detail/config.hpp>
-#include <boost/json/system_error.hpp>
 
 namespace boost {
 namespace json {
@@ -142,6 +141,9 @@ enum class error
     /// `double` was expected during conversion
     not_double,
 
+    /// JSON integer was expected during conversion
+    not_integer,
+
     /// source composite has size incompatible with target
     size_mismatch,
 
@@ -150,7 +152,6 @@ enum class error
 
     /// the key does not correspond to a known name
     unknown_name,
-
 };
 
 /** Error conditions corresponding to JSON errors
