@@ -14,6 +14,8 @@
 #include <boost/core/ignore_unused.hpp>
 #include <boost/describe/class.hpp>
 #include <boost/describe/enum.hpp>
+#include <boost/variant2/variant.hpp>
+#include <boost/config.hpp>
 
 #include "test_suite.hpp"
 
@@ -22,6 +24,14 @@
 #include <unordered_map>
 #include <vector>
 
+#ifndef BOOST_NO_CXX17_HDR_VARIANT
+# include <variant>
+#endif
+
+#ifndef BOOST_NO_CXX17_HDR_FILESYSTEM
+# include <filesystem>
+#endif // BOOST_NO_CXX17_HDR_FILESYSTEM
+
 namespace value_to_test_ns
 {
 
@@ -30,7 +40,7 @@ struct T1 { };
 //----------------------------------------------------------
 struct T2 { };
 
-boost::json::result<T2>
+boost::system::result<T2>
 tag_invoke(
     boost::json::try_value_to_tag<T2>,
     boost::json::value const& jv)
@@ -51,7 +61,7 @@ tag_invoke(
 {
     boost::json::string const* str = jv.if_string();
     if( !str )
-        throw boost::json::system_error(
+        throw boost::system::system_error(
             make_error_code(boost::json::error::not_string));
     if ( *str != "T3" )
         throw std::invalid_argument("");
@@ -109,8 +119,18 @@ BOOST_DESCRIBE_STRUCT(T6, (), (n, d))
 struct T7 : T6
 {
     std::string s;
+
+    bool
+    get_b() const
+    {
+        return b;
+    }
+
+private:
+    bool b = false;
+
+    BOOST_DESCRIBE_CLASS(T7, (T6), (s), (), (b))
 };
-BOOST_DESCRIBE_STRUCT(T7, (T6), (s))
 
 //----------------------------------------------------------
 
@@ -147,7 +167,7 @@ struct custom_context
 struct T9
 { };
 
-boost::json::result<T9>
+boost::system::result<T9>
 tag_invoke(
     boost::json::try_value_to_tag<T9>,
     boost::json::value const& jv,
@@ -159,6 +179,23 @@ tag_invoke(
     return make_error_code(boost::json::error::syntax);
 }
 
+// not default-constructible
+struct T11
+{
+    int n;
+
+    explicit T11( int v )
+        : n(v)
+    {}
+};
+T11
+tag_invoke(
+    boost::json::value_to_tag<T11>,
+    boost::json::value const& jv)
+{
+    return T11( jv.to_number<int>() );
+}
+
 } // namespace value_to_test_ns
 
 namespace std
@@ -394,17 +431,19 @@ public:
             BOOST_TEST( res->d == 0.125 );
 
             jv.as_object()["x"] = 0;
-            BOOST_TEST_THROWS_WITH_LOCATION(
-                value_to<::value_to_test_ns::T6>( jv ));
+            res = try_value_to<::value_to_test_ns::T6>(
+                jv, ctx... );
+            BOOST_TEST( res );
         }
         {
-            value jv = {{"n", 1}, {"d", 2}, {"s", "xyz"}};
+            value jv = {{"n", 1}, {"d", 2}, {"s", "xyz"}, {"b", true}};
             auto res = try_value_to<::value_to_test_ns::T7>(
                 jv, ctx... );
             BOOST_TEST( res );
             BOOST_TEST( res->n == 1 );
             BOOST_TEST( res->d == 2 );
             BOOST_TEST( res->s == "xyz" );
+            BOOST_TEST( res->get_b() == true );
         }
 
         BOOST_TEST_THROWS_WITH_LOCATION(
@@ -441,15 +480,15 @@ public:
             BOOST_TEST( std::nullopt == res->opt_s );
 
             jv.as_object()["x"] = 0;
-            BOOST_TEST_THROWS_WITH_LOCATION(
-                value_to<::value_to_test_ns::T8>( jv, ctx... ));
+            res = try_value_to<::value_to_test_ns::T8>(
+                jv, ctx... );
+            BOOST_TEST( res );
 #endif // BOOST_NO_CXX17_HDR_OPTIONAL
         }
 
-        BOOST_TEST_THROWS(
+        BOOST_TEST_THROWS_WITH_LOCATION(
             value_to<::value_to_test_ns::T10>(
-                value{{"n", 0}, {"t3", "t10"}}, ctx... ),
-            std::invalid_argument);
+                value{{"n", 0}, {"t3", "t10"}}, ctx... ));
 #endif // BOOST_DESCRIBE_CXX14
     }
 
@@ -470,24 +509,24 @@ public:
 #endif
     }
 
-    template< class... Context >
+    template<
+        template <class...> class Variant, class Monostate, class... Context >
     static
     void
     testVariant( Context const& ... ctx )
     {
-        ignore_unused( ctx... );
-#ifndef BOOST_NO_CXX17_HDR_VARIANT
-        using Var = std::variant<int, ::value_to_test_ns::T2, std::string>;
+        using std::get;
+        using Var = Variant<int, ::value_to_test_ns::T2, std::string>;
 
         value jv(4);
         auto v = value_to<Var>( jv, ctx... );
         BOOST_TEST( v.index() == 0 );
-        BOOST_TEST( std::get<0>(v) == 4 );
+        BOOST_TEST( get<0>(v) == 4 );
 
         jv = "foobar";
         v = value_to<Var>( jv, ctx... );
         BOOST_TEST( v.index() == 2 );
-        BOOST_TEST( std::get<2>(v) == "foobar" );
+        BOOST_TEST( get<2>(v) == "foobar" );
 
         jv = "T2";
         v = value_to<Var>( jv, ctx... );
@@ -497,10 +536,37 @@ public:
         BOOST_TEST_THROWS_WITH_LOCATION(
             value_to<Var>( jv, ctx... ));
 
-        value_to<std::monostate>( value(), ctx... );
+        value_to<Monostate>( value(), ctx... );
         BOOST_TEST_THROWS_WITH_LOCATION(
-            value_to<std::monostate>( jv, ctx... ));
-#endif // BOOST_NO_CXX17_HDR_VARIANT
+            value_to<Monostate>( jv, ctx... ));
+
+        jv = 1024;
+        using VT11 = Variant< value_to_test_ns::T11 >;
+        VT11 v11 = value_to< VT11 >( jv, ctx... );
+        BOOST_TEST( v11.index() == 0 );
+        BOOST_TEST( get<0>(v11).n == 1024 );
+
+        jv = nullptr;
+        using V_T3_T1 = Variant<value_to_test_ns::T3, value_to_test_ns::T1>;
+        auto v_t3_t1 = value_to<V_T3_T1>( jv, ctx... );
+        BOOST_TEST( v_t3_t1.index() == 1 );
+    }
+
+    template< class... Context >
+    static
+    void testPath( Context const& ... ctx )
+    {
+        ignore_unused( ctx... );
+#ifndef BOOST_NO_CXX17_HDR_FILESYSTEM
+        using Paths = std::vector<std::filesystem::path>;
+        value jv = value{"from/here", "to/there", "", "c:/" , "..", "../"};
+        auto paths = value_to<Paths>( jv, ctx... );
+        BOOST_TEST(
+            paths == (Paths{
+                "from/here", "to/there", "", "c:/" , "..", "../"}) );
+        BOOST_TEST_THROWS_WITH_LOCATION(
+            value_to<std::filesystem::path>( value(1), ctx... ));
+#endif // BOOST_NO_CXX17_HDR_FILESYSTEM
     }
 
     template< class... Context >
@@ -534,7 +600,7 @@ public:
         {
             BOOST_TEST_THROWS(
                 value_to<::value_to_test_ns::T2>( value(), ctx... ),
-                system_error);
+                system::system_error);
         }
         // nonthrowing overload falls back to throwing customization
         {
@@ -702,7 +768,7 @@ public:
         BOOST_TEST_THROWS(
             value_to<value_to_test_ns::T9>(
                 value(), value_to_test_ns::custom_context() ),
-            system_error);
+            system::system_error);
     }
 
     struct run_templated_tests
@@ -718,7 +784,11 @@ public:
             testNullptr( Context()... );
             testDescribed( Context()... );
             testOptional( Context()... );
-            testVariant( Context()... );
+            testVariant< variant2::variant, variant2::monostate > ( Context()... );
+#ifndef BOOST_NO_CXX17_HDR_VARIANT
+            testVariant< std::variant, std::monostate > ( Context()... );
+#endif // BOOST_NO_CXX17_HDR_VARIANT
+            testPath( Context()... );
             testNonThrowing( Context()... );
             testUserConversion( Context()... );
         }
