@@ -14,6 +14,7 @@
 
 #include "boostLocale/test/tools.hpp"
 #include "boostLocale/test/unit_test.hpp"
+#include "formatting_common.hpp"
 
 template<typename CharType, typename RefCharType>
 void test_by_char(const std::locale& l, const std::locale& lreal)
@@ -189,11 +190,11 @@ void test_main(int /*argc*/, char** /*argv*/)
             test_by_char<wchar_t, wchar_t>(l1, l2);
 
 #ifdef BOOST_LOCALE_ENABLE_CHAR16_T
-            std::cout << "\tchar16 UTF-16" << std::endl;
+            std::cout << "\tchar16_t" << std::endl;
             test_by_char<char16_t, char16_t>(l1, l2);
 #endif
 #ifdef BOOST_LOCALE_ENABLE_CHAR32_T
-            std::cout << "\tchar32 UTF-32" << std::endl;
+            std::cout << "\tchar32_t" << std::endl;
             test_by_char<char32_t, char32_t>(l1, l2);
 #endif
         }
@@ -230,6 +231,12 @@ void test_main(int /*argc*/, char** /*argv*/)
             }
         }
     }
+    // Std backend silently falls back to the C locale when the locale is not supported
+    // which breaks the test assumptions
+    if(has_std_locale("en_US.UTF-8")) {
+        test_format_large_number();
+        test_parse_multi_number();
+    }
 }
 
 // boostinspect:noascii
