@@ -1,5 +1,5 @@
 /*
- *          Copyright Andrey Semashev 2007 - 2015.
+ *          Copyright Andrey Semashev 2007 - 2025.
  * Distributed under the Boost Software License, Version 1.0.
  *    (See accompanying file LICENSE_1_0.txt or copy at
  *          http://www.boost.org/LICENSE_1_0.txt)
@@ -30,7 +30,6 @@
 #include <algorithm>
 #include <stdexcept>
 #include <boost/core/ref.hpp>
-#include <boost/bind/bind.hpp>
 #include <boost/cstdint.hpp>
 #include <boost/optional/optional.hpp>
 #include <boost/smart_ptr/make_shared_object.hpp>
@@ -44,7 +43,6 @@
 #include <boost/filesystem/path.hpp>
 #include <boost/filesystem/fstream.hpp>
 #include <boost/filesystem/operations.hpp>
-#include <boost/filesystem/convenience.hpp>
 #include <boost/intrusive/list.hpp>
 #include <boost/intrusive/list_hook.hpp>
 #include <boost/intrusive/options.hpp>
@@ -60,8 +58,7 @@
 #include "unique_ptr.hpp"
 
 #if !defined(BOOST_LOG_NO_THREADS)
-#include <boost/thread/locks.hpp>
-#include <boost/thread/mutex.hpp>
+#include <mutex>
 #endif // !defined(BOOST_LOG_NO_THREADS)
 
 #include <boost/log/detail/header.hpp>
@@ -510,7 +507,14 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
     }
 
     //! The function parses file name pattern and splits it into path and filename and creates a function object that will generate the actual filename from the pattern
-    void parse_file_name_pattern(filesystem::path const& pattern, filesystem::path& storage_dir, filesystem::path& file_name_pattern, boost::log::aux::light_function< path_string_type (unsigned int) >& file_name_generator)
+    void parse_file_name_pattern
+    (
+        filesystem::path const& pattern,
+        filesystem::path& storage_dir,
+        filesystem::path& file_name_pattern,
+        boost::log::aux::light_function< path_string_type (unsigned int) >& file_name_generator,
+        bool& has_file_counter
+    )
     {
         // Note: avoid calling Boost.Filesystem functions that involve path::codecvt()
         // https://svn.boost.org/trac/boost/ticket/9119
@@ -569,25 +573,48 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
             if (counter_found)
             {
                 // Both counter and date/time placeholder in the pattern
-                file_name_generator = boost::bind(date_and_time_formatter(),
-                    boost::bind(file_counter_formatter(counter_pos, width), name_pattern, boost::placeholders::_1), boost::placeholders::_1);
+#if !defined(BOOST_NO_CXX14_INITIALIZED_LAMBDA_CAPTURES)
+                file_name_generator = [date_and_time_fmt = date_and_time_formatter(), file_counter_fmt = file_counter_formatter(counter_pos, width), name_pattern](unsigned int counter)
+                    { return date_and_time_fmt(file_counter_fmt(name_pattern, counter), counter); };
+#else
+                date_and_time_formatter date_and_time_fmt;
+                file_counter_formatter file_counter_fmt(counter_pos, width);
+                file_name_generator = [date_and_time_fmt, file_counter_fmt, name_pattern](unsigned int counter)
+                    { return date_and_time_fmt(file_counter_fmt(name_pattern, counter), counter); };
+#endif
             }
             else
             {
                 // Only date/time placeholders in the pattern
-                file_name_generator = boost::bind(date_and_time_formatter(), name_pattern, boost::placeholders::_1);
+#if !defined(BOOST_NO_CXX14_INITIALIZED_LAMBDA_CAPTURES)
+                file_name_generator = [date_and_time_fmt = date_and_time_formatter(), name_pattern](unsigned int counter)
+                    { return date_and_time_fmt(name_pattern, counter); };
+#else
+                date_and_time_formatter date_and_time_fmt;
+                file_name_generator = [date_and_time_fmt, name_pattern](unsigned int counter)
+                    { return date_and_time_fmt(name_pattern, counter); };
+#endif
             }
         }
         else if (counter_found)
         {
             // Only counter placeholder in the pattern
-            file_name_generator = boost::bind(file_counter_formatter(counter_pos, width), name_pattern, boost::placeholders::_1);
+#if !defined(BOOST_NO_CXX14_INITIALIZED_LAMBDA_CAPTURES)
+            file_name_generator = [file_counter_fmt = file_counter_formatter(counter_pos, width), name_pattern](unsigned int counter)
+                { return file_counter_fmt(name_pattern, counter); };
+#else
+            file_counter_formatter file_counter_fmt(counter_pos, width);
+            file_name_generator = [file_counter_fmt, name_pattern](unsigned int counter)
+                { return file_counter_fmt(name_pattern, counter); };
+#endif
         }
         else
         {
             // No placeholders detected
             file_name_generator = empty_formatter(name_pattern);
         }
+
+        has_file_counter = counter_found;
     }
 
 
@@ -655,7 +682,7 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
 
 #if !defined(BOOST_LOG_NO_THREADS)
         //! Synchronization mutex
-        mutex m_Mutex;
+        std::mutex m_Mutex;
 #endif // !defined(BOOST_LOG_NO_THREADS)
 
         //! Total file size upper limit
@@ -746,7 +773,7 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
     private:
 #if !defined(BOOST_LOG_NO_THREADS)
         //! Synchronization mutex
-        mutex m_Mutex;
+        std::mutex m_Mutex;
 #endif // !defined(BOOST_LOG_NO_THREADS)
         //! The list of file collectors
         file_collectors m_Collectors;
@@ -845,7 +872,7 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
             filesystem::create_directories(m_StorageDir);
         }
 
-        BOOST_LOG_EXPR_IF_MT(lock_guard< mutex > lock(m_Mutex);)
+        BOOST_LOG_EXPR_IF_MT(std::lock_guard< std::mutex > lock(m_Mutex);)
 
         file_list::iterator it = m_Files.begin();
         const file_list::iterator end = m_Files.end();
@@ -963,7 +990,7 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
             filesystem::file_status status = filesystem::status(dir, ec);
             if (status.type() == filesystem::directory_file)
             {
-                BOOST_LOG_EXPR_IF_MT(lock_guard< mutex > lock(m_Mutex);)
+                BOOST_LOG_EXPR_IF_MT(std::lock_guard< std::mutex > lock(m_Mutex);)
 
                 file_list files;
                 filesystem::directory_iterator it(dir), end;
@@ -1012,7 +1039,7 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
     //! The function updates storage restrictions
     void file_collector::update(uintmax_t max_size, uintmax_t min_free_space, uintmax_t max_files)
     {
-        BOOST_LOG_EXPR_IF_MT(lock_guard< mutex > lock(m_Mutex);)
+        BOOST_LOG_EXPR_IF_MT(std::lock_guard< std::mutex > lock(m_Mutex);)
 
         m_MaxSize = (std::min)(m_MaxSize, max_size);
         m_MinFreeSpace = (std::max)(m_MinFreeSpace, min_free_space);
@@ -1024,10 +1051,10 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
     shared_ptr< file::collector > file_collector_repository::get_collector(
         filesystem::path const& target_dir, uintmax_t max_size, uintmax_t min_free_space, uintmax_t max_files)
     {
-        BOOST_LOG_EXPR_IF_MT(lock_guard< mutex > lock(m_Mutex);)
+        BOOST_LOG_EXPR_IF_MT(std::lock_guard< std::mutex > lock(m_Mutex);)
 
         file_collectors::iterator it = std::find_if(m_Collectors.begin(), m_Collectors.end(),
-            boost::bind(&file_collector::is_governed, boost::placeholders::_1, boost::cref(target_dir)));
+            [&target_dir](file_collector const& collector) { return collector.is_governed(target_dir); });
         shared_ptr< file_collector > p;
         if (it != m_Collectors.end()) try
         {
@@ -1052,7 +1079,7 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
     //! Removes the file collector from the list
     void file_collector_repository::remove_collector(file_collector* p)
     {
-        BOOST_LOG_EXPR_IF_MT(lock_guard< mutex > lock(m_Mutex);)
+        BOOST_LOG_EXPR_IF_MT(std::lock_guard< std::mutex > lock(m_Mutex);)
         m_Collectors.erase(m_Collectors.iterator_to(*p));
     }
 
@@ -1253,6 +1280,8 @@ struct text_file_backend::implementation
     filesystem::path m_StorageDir;
     //! File name generator (according to m_FileNamePattern)
     boost::log::aux::light_function< path_string_type (unsigned int) > m_FileNameGenerator;
+    //! The flag indicates whether m_FileNamePattern has a file counter placeholder
+    bool m_FileNamePatternHasCounter;
 
     //! Target file name pattern
     filesystem::path m_TargetFileNamePattern;
@@ -1298,6 +1327,7 @@ struct text_file_backend::implementation
     bool m_IsFirstFile;
 
     implementation(uintmax_t rotation_size, auto_newline_mode auto_newline, bool auto_flush, bool enable_final_rotation) :
+        m_FileNamePatternHasCounter(false),
         m_FileCounter(0u),
         m_FileOpenMode(std::ios_base::trunc | std::ios_base::out),
         m_CharactersWritten(0u),
@@ -1431,11 +1461,11 @@ BOOST_LOG_API void text_file_backend::consume(record_view const& rec, string_typ
             unsigned int file_counter = m_pImpl->m_FileCounter;
             if (BOOST_LIKELY(m_pImpl->m_FileCounterIsLastUsed))
             {
-                // If the sink backend is configured to append to a previously written file, don't
-                // increment the file counter and try to open the existing file. Only do this if the
-                // file is not moved to a different storage location by the file collector.
+                // If the sink backend is configured to append to a previously written file and the file pattern
+                // includes a file counter, don't increment the counter and try to open the existing file, with the last
+                // used counter value. Only do this if the file is not moved to a different storage location by the file collector.
                 bool increment_file_counter = true;
-                if (BOOST_UNLIKELY(m_pImpl->m_IsFirstFile && (m_pImpl->m_FileOpenMode & std::ios_base::app) != 0))
+                if (BOOST_UNLIKELY(m_pImpl->m_IsFirstFile && (m_pImpl->m_FileOpenMode & std::ios_base::app) != 0 && m_pImpl->m_FileNamePatternHasCounter))
                 {
                     filesystem::path last_file_name = m_pImpl->m_StorageDir / m_pImpl->m_FileNameGenerator(file_counter);
                     if (!!m_pImpl->m_pFileCollector)
@@ -1557,7 +1587,8 @@ BOOST_LOG_API void text_file_backend::set_file_name_pattern_internal(filesystem:
         !pattern.empty() ? pattern : filesystem::path(traits_t::default_file_name_pattern()),
         m_pImpl->m_StorageDir,
         m_pImpl->m_FileNamePattern,
-        m_pImpl->m_FileNameGenerator
+        m_pImpl->m_FileNameGenerator,
+        m_pImpl->m_FileNamePatternHasCounter
     );
 }
 
@@ -1566,7 +1597,8 @@ BOOST_LOG_API void text_file_backend::set_target_file_name_pattern_internal(file
 {
     if (!pattern.empty())
     {
-        parse_file_name_pattern(pattern, m_pImpl->m_TargetStorageDir, m_pImpl->m_TargetFileNamePattern, m_pImpl->m_TargetFileNameGenerator);
+        bool has_file_counter = false;
+        parse_file_name_pattern(pattern, m_pImpl->m_TargetStorageDir, m_pImpl->m_TargetFileNamePattern, m_pImpl->m_TargetFileNameGenerator, has_file_counter);
     }
     else
     {
