@@ -573,6 +573,37 @@ void test_complex_exponential_integral_E1(){
     }
 }
 
+template <class T>
+void test_non_central_t()
+{
+   //
+   // Bug case from the non-central t distribution:
+   //
+   using std::pow;
+   using std::exp;
+   using std::sqrt;
+
+   std::cout << "Testing non-central T PDF integral" << std::endl;
+
+   T x = -1.882352352142334;
+   T v = 77.384613037109375;
+   T mu = 8.1538467407226562;
+   T expected = static_cast<T>(4.5098555913703146875364186893655197e+49L);
+
+   boost::math::quadrature::exp_sinh<T> integrator;
+   T err;
+   T L1;
+   std::size_t levels;
+   T integral = integrator.integrate([&x, v, mu](T y)
+      {
+         return pow(y, v) * exp(boost::math::pow<2>((y - mu * x / sqrt(x * x + v))) / -2);
+      },
+      boost::math::tools::root_epsilon<T>(), &err, &L1, &levels);
+
+   T tol = 100 * boost::math::tools::epsilon<T>();
+   BOOST_CHECK_CLOSE_FRACTION(integral, expected, tol);
+}
+
 
 BOOST_AUTO_TEST_CASE(exp_sinh_quadrature_test)
 {
@@ -596,11 +627,13 @@ BOOST_AUTO_TEST_CASE(exp_sinh_quadrature_test)
     test_right_limit_infinite<std::float32_t>();
     test_nr_examples<std::float32_t>();
     test_crc<std::float32_t>();
+    //test_non_central_t<float32_t>();
 #else
     test_left_limit_infinite<float>();
     test_right_limit_infinite<float>();
     test_nr_examples<float>();
     test_crc<float>();
+    //test_non_central_t<float>();
 #endif
 
 #endif
@@ -611,11 +644,13 @@ BOOST_AUTO_TEST_CASE(exp_sinh_quadrature_test)
     test_right_limit_infinite<std::float64_t>();
     test_nr_examples<std::float64_t>();
     test_crc<std::float64_t>();
+    test_non_central_t<std::float64_t>();
 #else
     test_left_limit_infinite<double>();
     test_right_limit_infinite<double>();
     test_nr_examples<double>();
     test_crc<double>();
+    test_non_central_t<double>();
 #endif
 
 #endif
@@ -626,6 +661,7 @@ BOOST_AUTO_TEST_CASE(exp_sinh_quadrature_test)
     test_right_limit_infinite<long double>();
     test_nr_examples<long double>();
     test_crc<long double>();
+    test_non_central_t<long double>();
 #endif
 #endif
 #endif
@@ -642,6 +678,7 @@ BOOST_AUTO_TEST_CASE(exp_sinh_quadrature_test)
     test_right_limit_infinite<boost::math::concepts::real_concept>();
     test_nr_examples<boost::math::concepts::real_concept>();
     test_crc<boost::math::concepts::real_concept>();
+    test_non_central_t<boost::math::concepts::real_concept>();
 #endif
 #endif
 #if defined(TEST6) && defined(BOOST_MATH_RUN_MP_TESTS)
