@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2019-2023 Ruben Perez Hidalgo (rubenperez038 at gmail dot com)
+// Copyright (c) 2019-2025 Ruben Perez Hidalgo (rubenperez038 at gmail dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -11,18 +11,28 @@
 #include <boost/mysql/detail/ok_view.hpp>
 
 #include "test_unit/create_frame.hpp"
-#include "test_unit/serialization.hpp"
 
 namespace boost {
 namespace mysql {
 namespace test {
 
-inline std::vector<std::uint8_t> create_ok_body(const detail::ok_view& ok) { return serialize_ok(ok); }
-inline std::vector<std::uint8_t> create_eof_body(const detail::ok_view& ok) { return serialize_eof(ok); }
+std::vector<std::uint8_t> serialize_ok_impl(const detail::ok_view& pack, std::uint8_t header);
+
+inline std::vector<std::uint8_t> create_ok_body(const detail::ok_view& ok)
+{
+    return serialize_ok_impl(ok, 0x00);
+}
+
+inline std::vector<std::uint8_t> create_eof_body(const detail::ok_view& ok)
+{
+    return serialize_ok_impl(ok, 0xfe);
+}
+
 inline std::vector<std::uint8_t> create_ok_frame(std::uint8_t seqnum, const detail::ok_view& ok)
 {
     return create_frame(seqnum, create_ok_body(ok));
 }
+
 inline std::vector<std::uint8_t> create_eof_frame(std::uint8_t seqnum, const detail::ok_view& ok)
 {
     return create_frame(seqnum, create_eof_body(ok));
