@@ -1,11 +1,12 @@
 //
-// Copyright (c) 2019-2023 Ruben Perez Hidalgo (rubenperez038 at gmail dot com)
+// Copyright (c) 2019-2025 Ruben Perez Hidalgo (rubenperez038 at gmail dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
 
 #include <boost/mysql/datetime.hpp>
+#include <boost/mysql/string_view.hpp>
 
 #include <boost/test/unit_test.hpp>
 
@@ -21,11 +22,19 @@ using namespace boost::mysql::test;
 
 BOOST_AUTO_TEST_SUITE(test_datetime)
 
+// Helpers
 datetime from_timestamp(std::int64_t micros_since_epoch)
 {
     return datetime(datetime::time_point(datetime::time_point::duration(micros_since_epoch)));
 }
 
+#ifdef BOOST_MYSQL_HAS_LOCAL_TIME
+datetime from_local_timestamp(std::int64_t micros_since_epoch)
+{
+    return datetime(datetime::local_time_point(datetime::local_time_point::duration(micros_since_epoch)));
+}
+#endif
+
 BOOST_AUTO_TEST_CASE(default_ctor)
 {
     datetime d;
@@ -184,6 +193,23 @@ BOOST_AUTO_TEST_CASE(ctor_from_time_point_invalid)
     BOOST_CHECK_THROW(from_timestamp((std::numeric_limits<std::int64_t>::min)()), std::out_of_range);
 }
 
+#ifdef BOOST_MYSQL_HAS_LOCAL_TIME
+BOOST_AUTO_TEST_CASE(ctor_from_local_time_point)
+{
+    BOOST_TEST(from_local_timestamp(253402300799999999) == datetime(9999, 12, 31, 23, 59, 59, 999999));
+    BOOST_TEST(from_local_timestamp(1715966176806454) == datetime(2024, 5, 17, 17, 16, 16, 806454));
+    BOOST_TEST(from_local_timestamp(-62167219200000000) == datetime(0, 1, 1));
+}
+
+BOOST_AUTO_TEST_CASE(ctor_from_local_time_point_invalid)
+{
+    BOOST_CHECK_THROW(from_local_timestamp(253402300799999999 + 1), std::out_of_range);
+    BOOST_CHECK_THROW(from_local_timestamp(-62167219200000000 - 1), std::out_of_range);
+    BOOST_CHECK_THROW(from_local_timestamp((std::numeric_limits<std::int64_t>::max)()), std::out_of_range);
+    BOOST_CHECK_THROW(from_local_timestamp((std::numeric_limits<std::int64_t>::min)()), std::out_of_range);
+}
+#endif
+
 // spotcheck, uses the same routines as get_time_point
 BOOST_AUTO_TEST_CASE(as_time_point)
 {
@@ -194,6 +220,25 @@ BOOST_AUTO_TEST_CASE(as_time_point)
     BOOST_CHECK_THROW(d2.as_time_point(), std::invalid_argument);
 }
 
+#ifdef BOOST_MYSQL_HAS_LOCAL_TIME
+// spotcheck, uses the same routines as get_time_point
+BOOST_AUTO_TEST_CASE(as_local_time_point)
+{
+    datetime d1(2010, 12, 31, 23, 59, 59, 999999);
+    BOOST_TEST(d1.as_local_time_point().time_since_epoch().count() == 1293839999999999);
+
+    datetime d2(0xffff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xffffffff);
+    BOOST_CHECK_THROW(d2.as_local_time_point(), std::invalid_argument);
+}
+
+// spotcheck, uses the same routines as get_time_point
+BOOST_AUTO_TEST_CASE(get_local_time_point)
+{
+    datetime d(2010, 12, 31, 23, 59, 59, 999999);
+    BOOST_TEST(d.get_local_time_point().time_since_epoch().count() == 1293839999999999);
+}
+#endif
+
 BOOST_AUTO_TEST_CASE(operator_equals)
 {
     datetime maxdt(0xffff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xffffffff);
@@ -232,110 +277,17 @@ BOOST_AUTO_TEST_CASE(operator_equals)
     }
 }
 
+// operator<< is implemented in terms of datetime_to_string (see dt_to_string.hpp)
 BOOST_AUTO_TEST_CASE(operator_stream)
 {
-    // Helper struct to define stream operations for date, datetime and time
-    // We will list the possibilities for each component (hours, minutes, days...) and will
-    // take the Cartessian product of all them
-    struct component_value
-    {
-        const char* name;
-        unsigned v;
-        const char* repr;
-    };
-
-    constexpr component_value year_values[] = {
-        {"min",       0,      "0000" },
-        {"onedig",    1,      "0001" },
-        {"twodig",    98,     "0098" },
-        {"threedig",  789,    "0789" },
-        {"regular",   1999,   "1999" },
-        {"max_mysql", 9999,   "9999" },
-        {"max",       0xffff, "65535"},
-    };
-
-    constexpr component_value month_values[] = {
-        {"zero",   0,    "00" },
-        {"onedig", 2,    "02" },
-        {"twodig", 12,   "12" },
-        {"max",    0xff, "255"},
-    };
-
-    constexpr component_value day_values[] = {
-        {"zero",   0,    "00" },
-        {"onedig", 1,    "01" },
-        {"twodig", 31,   "31" },
-        {"max",    0xff, "255"},
-    };
-
-    constexpr component_value hours_values[] = {
-        {"zero",   0,    "00" },
-        {"onedig", 5,    "05" },
-        {"twodig", 23,   "23" },
-        {"max",    0xff, "255"},
-    };
-
-    constexpr component_value mins_secs_values[] = {
-        {"zero",   0,    "00" },
-        {"onedig", 5,    "05" },
-        {"twodig", 59,   "59" },
-        {"max",    0xff, "255"},
-    };
-
-    constexpr component_value micros_values[] = {
-        {"zero",      0,          "000000"    },
-        {"onedig",    5,          "000005"    },
-        {"twodig",    50,         "000050"    },
-        {"max_mysql", 999999,     "999999"    },
-        {"max",       0xffffffff, "4294967295"},
-    };
-
-    // clang-format off
-    for (const auto& year : year_values)
-    {
-    for (const auto& month : month_values)
-    {
-    for (const auto& day : day_values)
-    {
-    for (const auto& hours : hours_values)
-    {
-    for (const auto& mins : mins_secs_values)
-    {
-    for (const auto& secs : mins_secs_values)
-    {
-    for (const auto& micros : micros_values)
-    {
-        BOOST_TEST_CONTEXT(
-            "year=" << year.name << ", month=" << month.name << "day=" << day.name <<
-            "hour=" << hours.name << ", mins=" << mins.name << ", secs=" << secs.name <<
-            "micros=" << micros.name
-        )
-        {
-            std::string str_val = stringize(
-                year.repr, '-', month.repr, '-', day.repr, ' ',
-                hours.repr, ':', mins.repr, ':', secs.repr,
-                '.', micros.repr
-            );
-            datetime dt(
-                static_cast<std::uint16_t>(year.v),
-                static_cast<std::uint8_t>(month.v),
-                static_cast<std::uint8_t>(day.v),
-                static_cast<std::uint8_t>(hours.v),
-                static_cast<std::uint8_t>(mins.v),
-                static_cast<std::uint8_t>(secs.v),
-                static_cast<std::uint32_t>(micros.v)
-            );
-
-            BOOST_TEST(stringize(dt) == str_val);
-        }
-    }
-    }
-    }
-    }
-    }
-    }
-    }
-    // clang-format on
+    BOOST_TEST(stringize(datetime(2023, 1, 2, 12, 10, 1, 0)) == "2023-01-02 12:10:01.000000");
+    BOOST_TEST(stringize(datetime(2022, 12, 31)) == "2022-12-31 00:00:00.000000");
+    BOOST_TEST(stringize(datetime(2020, 3, 2, 23, 59, 59, 12345)) == "2020-03-02 23:59:59.012345");
+    BOOST_TEST(stringize(datetime()) == "0000-00-00 00:00:00.000000");
+    BOOST_TEST(
+        stringize(datetime(0xffff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xffffffff)) ==
+        "65535-255-255 255:255:255.4294967295"
+    );
 }
 
 BOOST_AUTO_TEST_CASE(now)
