@@ -13,7 +13,7 @@
 #include<string>
 #include "boost/optional/optional.hpp"
 
-#ifdef __BORLANDC__
+#ifdef BOOST_BORLANDC
 #pragma hdrstop
 #endif
 
@@ -31,10 +31,10 @@ struct Guard
   std::string str;
   Guard() : num() {}
   Guard(double num_, std::string str_) : num(num_), str(str_) {}
-  
+
   friend bool operator==(const Guard& lhs, const Guard& rhs) { return lhs.num == rhs.num && lhs.str == rhs.str; }
   friend bool operator!=(const Guard& lhs, const Guard& rhs) { return !(lhs == rhs); }
-  
+
 private:
   Guard(const Guard&);
   Guard& operator=(const Guard&);
@@ -50,4 +50,4 @@ int main()
   NOTHING_TO_TEST_SO_JUST_FAIL
 #endif
   return 0;
-}
\ No newline at end of file
+}
