@@ -7,9 +7,9 @@ echo "=================================================================="
 echo "."
 echo "C O M P I L I N G . . . . . . . . . . ."
 
-g++ ./file_generator.cpp -std=c++11 -march=native -w -fexceptions -O3 -I../../include  -s  -o file_generator
+g++ ./file_generator.cpp -std=c++11 -march=native -w -fno-exceptions -fno-operator-names -O3 -I../../include  -s  -o file_generator
 
-g++ ./benchmark_objects.cpp -std=c++11 -march=native -w -fexceptions -O3 -I../../include -s  -o benchmark_objects
+g++ ./benchmark_objects.cpp -std=c++11 -march=native -w -fno-exceptions -fno-operator-names -O3 -I../../include -s  -o benchmark_objects
 echo "."
 echo "R U N N I N G . . . . . . . . . . ."
 echo "( The time needed is around 45 minutes depending of your machine )"
