@@ -976,7 +976,7 @@ int main()
     // GCC's std::to_address() breaks the contiguous_iterator concept.
     // https://gcc.gnu.org/bugzilla/show_bug.cgi?id=96416
 #if !BOOST_STL_INTERFACES_USE_CONCEPTS ||                                      \
-    defined(__GNUC__) && __GNUC__ >= 11 && __GNUC_MINOR__ >= 3
+    defined(__GNUC__) && __GNUC__ == 11 && __GNUC_MINOR__ >= 3
     // data
     {
         BOOST_TEST(r.data() != nullptr);
