@@ -1,6 +1,6 @@
 
 // Copyright 2005-2009 Daniel James.
-// Copyright 2022 Christian Mazakas.
+// Copyright 2022-2023 Christian Mazakas.
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -18,15 +18,16 @@
 #endif
 
 #include "../helpers/check_return_type.hpp"
+#include <boost/core/invoke_swap.hpp>
 #include <boost/core/pointer_traits.hpp>
 #include <boost/limits.hpp>
 #include <boost/predef.h>
 #include <boost/static_assert.hpp>
+#include <boost/tuple/tuple.hpp>
 #include <boost/type_traits/cv_traits.hpp>
 #include <boost/type_traits/is_const.hpp>
 #include <boost/type_traits/is_convertible.hpp>
 #include <boost/type_traits/is_same.hpp>
-#include <boost/utility/swap.hpp>
 
 typedef long double comparison_type;
 
@@ -34,22 +35,9 @@ template <class T> void sink(T const&) {}
 template <class T> T rvalue(T const& v) { return v; }
 template <class T> T rvalue_default() { return T(); }
 
-#if !defined(BOOST_NO_CXX11_HDR_INITIALIZER_LIST)
 template <class T> T implicit_construct() { return {}; }
-#else
-template <class T> int implicit_construct()
-{
-  T x;
-  sink(x);
-  return 0;
-}
-#endif
 
-#if !defined(BOOST_NO_CXX11_NOEXCEPT)
-#define TEST_NOEXCEPT_EXPR(x) BOOST_STATIC_ASSERT((BOOST_NOEXCEPT_EXPR(x)));
-#else
-#define TEST_NOEXCEPT_EXPR(x)
-#endif
+#define TEST_NOEXCEPT_EXPR(x) BOOST_STATIC_ASSERT((noexcept(x)));
 
 template <class X, class T> void container_test(X& r, T const&)
 {
@@ -143,13 +131,11 @@ template <class X, class T> void container_test(X& r, T const&)
 
 // I don't test the runtime post-conditions here.
 
-#if !defined(BOOST_NO_CXX11_HDR_INITIALIZER_LIST)
   // It isn't specified in the container requirements that the no argument
   // constructor is implicit, but it is defined that way in the concrete
   // container specification.
   X u_implicit = {};
   sink(u_implicit);
-#endif
 
   X u;
   BOOST_TEST(u.size() == 0);
@@ -165,7 +151,7 @@ template <class X, class T> void container_test(X& r, T const&)
   X u5 = rvalue(a_const);
 
   a.swap(b);
-  boost::swap(a, b);
+  boost::core::invoke_swap(a, b);
   test::check_return_type<X>::equals_ref(r = a);
 
   // Allocator
@@ -191,10 +177,10 @@ template <class X, class T> void container_test(X& r, T const&)
   node_type n1;
   node_type n2(rvalue_default<node_type>());
 #if !BOOST_COMP_GNUC || BOOST_COMP_GNUC >= BOOST_VERSION_NUMBER(4, 8, 0)
-  TEST_NOEXCEPT_EXPR(node_type(boost::move(n1)));
+  TEST_NOEXCEPT_EXPR(node_type(std::move(n1)));
 #endif
   node_type n3;
-  n3 = boost::move(n2);
+  n3 = std::move(n2);
   n1.swap(n3);
   swap(n1, n3);
   // TODO: noexcept for swap?
@@ -229,13 +215,11 @@ template <class X> void unordered_destructible_test(X&)
 
   X x1;
 
-#if !defined(BOOST_NO_CXX11_RVALUE_REFERENCES)
   X x2(rvalue_default<X>());
   X x3 = rvalue_default<X>();
-// This can only be done if propagate_on_container_move_assignment::value
-// is true.
-// x2 = rvalue_default<X>();
-#endif
+  // This can only be done if propagate_on_container_move_assignment::value
+  // is true.
+  // x2 = rvalue_default<X>();
 
   X* ptr = new X();
   X& a1 = *ptr;
@@ -254,7 +238,7 @@ template <class X> void unordered_destructible_test(X&)
   test::check_return_type<const_iterator>::equals(a_const.cend());
 
   a.swap(b);
-  boost::swap(a, b);
+  boost::core::invoke_swap(a, b);
 
   test::check_return_type<size_type>::equals(a.size());
   test::check_return_type<size_type>::equals(a.max_size());
@@ -350,7 +334,7 @@ template <class X, class Key> void unordered_set_test(X& r, Key const&)
 
   test::minimal::constructor_param v;
   Key k_lvalue(v);
-  r.emplace(boost::move(k_lvalue));
+  r.emplace(std::move(k_lvalue));
   node_type n1 = r.extract(r.begin());
   test::check_return_type<value_type>::equals_ref(n1.value());
 #endif
@@ -493,11 +477,11 @@ void unordered_map_test(X& r, Key const& k, T const& v)
   test::check_return_type<key_type>::equals_ref(n1.key());
   test::check_return_type<T>::equals_ref(n1.mapped());
 
-  node_type n2 = boost::move(n1);
-  r.insert(boost::move(n2));
+  node_type n2 = std::move(n1);
+  r.insert(std::move(n2));
   r.insert(r.extract(r.begin()));
   n2 = r.extract(r.begin());
-  r.insert(r.begin(), boost::move(n2));
+  r.insert(r.begin(), std::move(n2));
   r.insert(r.end(), r.extract(r.begin()));
 
   node_type n = r.extract(r.begin());
@@ -512,8 +496,6 @@ template <class X> void equality_test(X& r)
 
   test::check_return_type<bool>::equals(a == b);
   test::check_return_type<bool>::equals(a != b);
-  test::check_return_type<bool>::equals(boost::operator==(a, b));
-  test::check_return_type<bool>::equals(boost::operator!=(a, b));
 }
 
 template <class X, class T> void unordered_unique_test(X& r, T const& t)
@@ -530,11 +512,11 @@ template <class X, class T> void unordered_unique_test(X& r, T const& t)
 
   // TODO;
   // boost::function_requires<
-  //     boost::MoveConstructibleConcept<insert_return_type>
+  //     std::moveConstructibleConcept<insert_return_type>
   // >();
   // TODO;
   // boost::function_requires<
-  //     boost::MoveAssignableConcept<insert_return_type>
+  //     std::moveAssignableConcept<insert_return_type>
   // >();
   boost::function_requires<
     boost::DefaultConstructibleConcept<insert_return_type> >();
@@ -546,7 +528,7 @@ template <class X, class T> void unordered_unique_test(X& r, T const& t)
   test::check_return_type<bool>::equals(insert_return.inserted);
   test::check_return_type<iterator>::equals(insert_return.position);
   test::check_return_type<node_type>::equals_ref(insert_return.node);
-  boost::swap(insert_return, insert_return2);
+  boost::core::invoke_swap(insert_return, insert_return2);
 #endif
 }
 
@@ -738,9 +720,7 @@ void unordered_test(X& x, Key& k, Hash& hf, Pred& eq)
   a.rehash(100);
 
   a.merge(a2);
-#if !defined(BOOST_NO_CXX11_RVALUE_REFERENCES)
   a.merge(rvalue_default<X>());
-#endif
 
   // Avoid unused variable warnings:
 
@@ -793,7 +773,6 @@ void unordered_copyable_test(X& x, Key& k, T& t, Hash& hf, Pred& eq)
 // X a8a(i, j, m);
 // sink(a8a);
 
-#if !defined(BOOST_NO_CXX11_HDR_INITIALIZER_LIST)
   std::size_t min_buckets = 10;
   X({t});
   X({t}, min_buckets);
@@ -803,7 +782,6 @@ void unordered_copyable_test(X& x, Key& k, T& t, Hash& hf, Pred& eq)
   X({t}, min_buckets, m);
   X({t}, min_buckets, hf, m);
   X({t}, min_buckets, hf, eq, m);
-#endif
 
   X const b;
   sink(X(b));
@@ -826,7 +804,6 @@ void unordered_copyable_test(X& x, Key& k, T& t, Hash& hf, Pred& eq)
   test::check_return_type<iterator>::equals(a.emplace_hint(q, t));
 
   a.insert(i, j);
-#if !defined(BOOST_NO_CXX11_HDR_INITIALIZER_LIST)
   std::initializer_list<T> list = {t};
   a.insert(list);
   a.insert({t, t, t});
@@ -837,7 +814,6 @@ void unordered_copyable_test(X& x, Key& k, T& t, Hash& hf, Pred& eq)
   a.insert({});
   a.insert({t});
   a.insert({t, t});
-#endif
 #endif
 
   X a10;
@@ -879,12 +855,10 @@ void unordered_movable_test(X& x, Key& k, T& /* t */, Hash& hf, Pred& eq)
   typedef typename X::const_iterator const_iterator;
   typedef typename X::allocator_type allocator_type;
 
-#if !defined(BOOST_NO_CXX11_RVALUE_REFERENCES)
   X x1(rvalue_default<X>());
-  X x2(boost::move(x1));
+  X x2(std::move(x1));
   x1 = rvalue_default<X>();
-  x2 = boost::move(x1);
-#endif
+  x2 = std::move(x1);
 
   X a;
   allocator_type m = a.get_allocator();
@@ -922,19 +896,19 @@ void unordered_movable_test(X& x, Key& k, T& /* t */, Hash& hf, Pred& eq)
   test::check_return_type<iterator>::equals(a.emplace_hint(q, v));
 
   T v1(v);
-  a.emplace(boost::move(v1));
+  a.emplace(std::move(v1));
   T v2(v);
-  a.insert(boost::move(v2));
+  a.insert(std::move(v2));
   T v3(v);
-  test::check_return_type<iterator>::equals(a.emplace_hint(q, boost::move(v3)));
+  test::check_return_type<iterator>::equals(a.emplace_hint(q, std::move(v3)));
   T v4(v);
-  test::check_return_type<iterator>::equals(a.insert(q, boost::move(v4)));
+  test::check_return_type<iterator>::equals(a.insert(q, std::move(v4)));
 
   a.insert(i, j);
 
   X a10;
   T v5(v);
-  a10.insert(boost::move(v5));
+  a10.insert(std::move(v5));
   q = a10.cbegin();
 #ifdef BOOST_UNORDERED_FOA_TESTS
   test::check_return_type<iterator>::convertible(a10.erase(q));
