@@ -6,14 +6,19 @@
 
 #include "../helpers/unordered.hpp"
 
-#include <boost/functional/hash/hash.hpp>
-#include "../helpers/test.hpp"
 #include "../helpers/count.hpp"
+#include "../helpers/test.hpp"
+#include <boost/container_hash/hash.hpp>
+#include <boost/tuple/tuple.hpp>
 #include <string>
+#include <type_traits>
 
 // Test that various emplace methods work with different numbers of
 // arguments.
 
+BOOST_UNORDERED_STATIC_ASSERT(std::is_same<std::piecewise_construct_t,
+  boost::unordered::piecewise_construct_t>::value);
+
 namespace emplace_tests {
   // Constructible with 2 to 10 arguments
   struct emplace_value : private test::counted_object
@@ -179,18 +184,13 @@ namespace emplace_tests {
 
   };
 
-  UNORDERED_AUTO_TEST (emplace_set) {
+  template <class X> static void emplace_set(X*)
+  {
     test::check_instances check_;
 
-#ifdef BOOST_UNORDERED_FOA_TESTS
-    typedef boost::unordered_flat_set<emplace_value,
-      boost::hash<emplace_value> >
-      container;
-#else
-    typedef boost::unordered_set<emplace_value, boost::hash<emplace_value> >
-      container;
-#endif
-    typedef container::iterator iterator;
+    typedef X container;
+
+    typedef typename container::iterator iterator;
     typedef std::pair<iterator, bool> return_type;
     container x(10);
     iterator i1;
@@ -225,7 +225,7 @@ namespace emplace_tests {
     BOOST_TEST(r1.first == x.find(v2));
     BOOST_TEST_EQ(check_.instances(), 4);
 #ifdef BOOST_UNORDERED_FOA_TESTS
-  BOOST_TEST_EQ(check_.constructions(), 6);
+    BOOST_TEST_EQ(check_.constructions(), 6);
 #else
     BOOST_TEST_EQ(check_.constructions(), 4);
 #endif
@@ -294,6 +294,21 @@ namespace emplace_tests {
     BOOST_TEST(x.count(v4) == 1);
   }
 
+#ifdef BOOST_UNORDERED_FOA_TESTS
+  static boost::unordered_flat_set<emplace_value, boost::hash<emplace_value> >*
+    test_set;
+
+  static boost::unordered_node_set<emplace_value, boost::hash<emplace_value> >*
+    test_node_set;
+
+  UNORDERED_TEST(emplace_set, ((test_set)(test_node_set)))
+#else
+  static boost::unordered_set<emplace_value, boost::hash<emplace_value> >*
+    test_set;
+
+  UNORDERED_TEST(emplace_set, ((test_set)))
+#endif
+
 #ifndef BOOST_UNORDERED_FOA_TESTS
   UNORDERED_AUTO_TEST (emplace_multiset) {
     test::check_instances check_;
@@ -374,26 +389,22 @@ namespace emplace_tests {
   }
 #endif
 
-  UNORDERED_AUTO_TEST (emplace_map) {
-    test::check_instances check_;
+  template <class X> static void emplace_map(X*)
+  {
 #ifdef BOOST_UNORDERED_FOA_TESTS
-    typedef boost::unordered_flat_map<emplace_value, emplace_value,
-      boost::hash<emplace_value> >
-      container;
-#else
-    typedef boost::unordered_map<emplace_value, emplace_value,
-      boost::hash<emplace_value> >
-      container;
-#endif
-    typedef container::iterator iterator;
+    test::check_instances check_;
+
+    typedef X container;
+    typedef typename container::iterator iterator;
     typedef std::pair<iterator, bool> return_type;
     container x(10);
     return_type r1, r2;
 
-#ifdef BOOST_UNORDERED_FOA_TESTS
     // 5/8 args + duplicate
     emplace_value k1(5, "", 'b', 4, 5);
+    BOOST_TEST_EQ(check_.constructions(), 1);
     emplace_value m1(8, "xxx", 'z', 4, 5, 6, 7, 8);
+    BOOST_TEST_EQ(check_.constructions(), 2);
     r1 = x.emplace(std::piecewise_construct, std::make_tuple(5, "", 'b', 4, 5),
       std::make_tuple(8, "xxx", 'z', 4, 5, 6, 7, 8));
     BOOST_TEST_EQ(x.size(), 1u);
@@ -444,66 +455,156 @@ namespace emplace_tests {
     BOOST_TEST_EQ(check_.instances(), 8);
     BOOST_TEST_EQ(check_.constructions(), 20);
 #else
-    // 5/8 args + duplicate
-    emplace_value k1(5, "", 'b', 4, 5);
-    emplace_value m1(8, "xxx", 'z', 4, 5, 6, 7, 8);
-    r1 = x.emplace(boost::unordered::piecewise_construct,
-      boost::make_tuple(5, "", 'b', 4, 5),
-      boost::make_tuple(8, "xxx", 'z', 4, 5, 6, 7, 8));
-    BOOST_TEST_EQ(x.size(), 1u);
-    BOOST_TEST(r1.second);
-    BOOST_TEST(x.find(k1) == r1.first);
-    BOOST_TEST(x.find(k1)->second == m1);
-    BOOST_TEST_EQ(check_.instances(), 4);
-    BOOST_TEST_EQ(check_.constructions(), 4);
+    {
+      test::check_instances check_;
 
-    r2 = x.emplace(boost::unordered::piecewise_construct,
-      boost::make_tuple(5, "", 'b', 4, 5),
-      boost::make_tuple(8, "xxx", 'z', 4, 5, 6, 7, 8));
-    BOOST_TEST_EQ(x.size(), 1u);
-    BOOST_TEST(!r2.second);
-    BOOST_TEST(r1.first == r2.first);
-    BOOST_TEST(x.find(k1)->second == m1);
-    BOOST_TEST_EQ(check_.instances(), 4);
-    // constructions could possibly be 5 if the implementation only
-    // constructed the key.
-    BOOST_TEST_EQ(check_.constructions(), 6);
+      typedef X container;
+      typedef typename container::iterator iterator;
+      typedef std::pair<iterator, bool> return_type;
+      container x(10);
+      return_type r1, r2;
 
-    // 9/3 args + duplicates with hints, different mapped value.
+      // 5/8 args + duplicate
+      emplace_value k1(5, "", 'b', 4, 5);
+      emplace_value m1(8, "xxx", 'z', 4, 5, 6, 7, 8);
+      r1 =
+        x.emplace(std::piecewise_construct, std::make_tuple(5, "", 'b', 4, 5),
+          std::make_tuple(8, "xxx", 'z', 4, 5, 6, 7, 8));
+      BOOST_TEST_EQ(x.size(), 1u);
+      BOOST_TEST(r1.second);
+      BOOST_TEST(x.find(k1) == r1.first);
+      BOOST_TEST(x.find(k1)->second == m1);
+      BOOST_TEST_EQ(check_.instances(), 4);
+      BOOST_TEST_EQ(check_.constructions(), 4);
 
-    emplace_value k2(9, "", 'b', 4, 5, 6, 7, 8, 9);
-    emplace_value m2(3, "aaa", 'm');
-    r1 = x.emplace(boost::unordered::piecewise_construct,
-      boost::make_tuple(9, "", 'b', 4, 5, 6, 7, 8, 9),
-      boost::make_tuple(3, "aaa", 'm'));
-    BOOST_TEST_EQ(x.size(), 2u);
-    BOOST_TEST(r1.second);
-    BOOST_TEST(r1.first->first.arg_count == 9);
-    BOOST_TEST(r1.first->second.arg_count == 3);
-    BOOST_TEST(x.find(k2) == r1.first);
-    BOOST_TEST(x.find(k2)->second == m2);
-    BOOST_TEST_EQ(check_.instances(), 8);
-    BOOST_TEST_EQ(check_.constructions(), 10);
+      r2 =
+        x.emplace(std::piecewise_construct, std::make_tuple(5, "", 'b', 4, 5),
+          std::make_tuple(8, "xxx", 'z', 4, 5, 6, 7, 8));
+      BOOST_TEST_EQ(x.size(), 1u);
+      BOOST_TEST(!r2.second);
+      BOOST_TEST(r1.first == r2.first);
+      BOOST_TEST(x.find(k1)->second == m1);
+      BOOST_TEST_EQ(check_.instances(), 4);
+      // constructions could possibly be 5 if the implementation only
+      // constructed the key.
+      BOOST_TEST_EQ(check_.constructions(), 6);
 
-    BOOST_TEST(r1.first ==
-               x.emplace_hint(r1.first, boost::unordered::piecewise_construct,
-                 boost::make_tuple(9, "", 'b', 4, 5, 6, 7, 8, 9),
-                 boost::make_tuple(15, "jkjk")));
-    BOOST_TEST(r1.first ==
-               x.emplace_hint(r2.first, boost::unordered::piecewise_construct,
-                 boost::make_tuple(9, "", 'b', 4, 5, 6, 7, 8, 9),
-                 boost::make_tuple(275, "xxx", 'm', 6)));
-    BOOST_TEST(r1.first ==
-               x.emplace_hint(x.end(), boost::unordered::piecewise_construct,
-                 boost::make_tuple(9, "", 'b', 4, 5, 6, 7, 8, 9),
-                 boost::make_tuple(-10, "blah blah", '\0')));
-    BOOST_TEST_EQ(x.size(), 2u);
-    BOOST_TEST(x.find(k2)->second == m2);
-    BOOST_TEST_EQ(check_.instances(), 8);
-    BOOST_TEST_EQ(check_.constructions(), 16);
+      // 9/3 args + duplicates with hints, different mapped value.
+
+      emplace_value k2(9, "", 'b', 4, 5, 6, 7, 8, 9);
+      emplace_value m2(3, "aaa", 'm');
+      r1 = x.emplace(std::piecewise_construct,
+        std::make_tuple(9, "", 'b', 4, 5, 6, 7, 8, 9),
+        std::make_tuple(3, "aaa", 'm'));
+      BOOST_TEST_EQ(x.size(), 2u);
+      BOOST_TEST(r1.second);
+      BOOST_TEST(r1.first->first.arg_count == 9);
+      BOOST_TEST(r1.first->second.arg_count == 3);
+      BOOST_TEST(x.find(k2) == r1.first);
+      BOOST_TEST(x.find(k2)->second == m2);
+      BOOST_TEST_EQ(check_.instances(), 8);
+      BOOST_TEST_EQ(check_.constructions(), 10);
+
+      BOOST_TEST(r1.first == x.emplace_hint(r1.first, std::piecewise_construct,
+                               std::make_tuple(9, "", 'b', 4, 5, 6, 7, 8, 9),
+                               std::make_tuple(15, "jkjk")));
+      BOOST_TEST(r1.first == x.emplace_hint(r2.first, std::piecewise_construct,
+                               std::make_tuple(9, "", 'b', 4, 5, 6, 7, 8, 9),
+                               std::make_tuple(275, "xxx", 'm', 6)));
+      BOOST_TEST(r1.first == x.emplace_hint(x.end(), std::piecewise_construct,
+                               std::make_tuple(9, "", 'b', 4, 5, 6, 7, 8, 9),
+                               std::make_tuple(-10, "blah blah", '\0')));
+      BOOST_TEST_EQ(x.size(), 2u);
+      BOOST_TEST(x.find(k2)->second == m2);
+      BOOST_TEST_EQ(check_.instances(), 8);
+      BOOST_TEST_EQ(check_.constructions(), 16);
+    }
+
+    {
+      test::check_instances check_;
+
+      typedef X container;
+      typedef typename container::iterator iterator;
+      typedef std::pair<iterator, bool> return_type;
+      container x(10);
+      return_type r1, r2;
+
+      // 5/8 args + duplicate
+      emplace_value k1(5, "", 'b', 4, 5);
+      emplace_value m1(8, "xxx", 'z', 4, 5, 6, 7, 8);
+      r1 =
+        x.emplace(boost::unordered::piecewise_construct, boost::make_tuple(5, "", 'b', 4, 5),
+          boost::make_tuple(8, "xxx", 'z', 4, 5, 6, 7, 8));
+      BOOST_TEST_EQ(x.size(), 1u);
+      BOOST_TEST(r1.second);
+      BOOST_TEST(x.find(k1) == r1.first);
+      BOOST_TEST(x.find(k1)->second == m1);
+      BOOST_TEST_EQ(check_.instances(), 4);
+      BOOST_TEST_EQ(check_.constructions(), 4);
+
+      r2 =
+        x.emplace(boost::unordered::piecewise_construct, boost::make_tuple(5, "", 'b', 4, 5),
+          boost::make_tuple(8, "xxx", 'z', 4, 5, 6, 7, 8));
+      BOOST_TEST_EQ(x.size(), 1u);
+      BOOST_TEST(!r2.second);
+      BOOST_TEST(r1.first == r2.first);
+      BOOST_TEST(x.find(k1)->second == m1);
+      BOOST_TEST_EQ(check_.instances(), 4);
+      // constructions could possibly be 5 if the implementation only
+      // constructed the key.
+      BOOST_TEST_EQ(check_.constructions(), 6);
+
+      // 9/3 args + duplicates with hints, different mapped value.
+
+      emplace_value k2(9, "", 'b', 4, 5, 6, 7, 8, 9);
+      emplace_value m2(3, "aaa", 'm');
+      r1 = x.emplace(boost::unordered::piecewise_construct,
+        boost::make_tuple(9, "", 'b', 4, 5, 6, 7, 8, 9),
+        boost::make_tuple(3, "aaa", 'm'));
+      BOOST_TEST_EQ(x.size(), 2u);
+      BOOST_TEST(r1.second);
+      BOOST_TEST(r1.first->first.arg_count == 9);
+      BOOST_TEST(r1.first->second.arg_count == 3);
+      BOOST_TEST(x.find(k2) == r1.first);
+      BOOST_TEST(x.find(k2)->second == m2);
+      BOOST_TEST_EQ(check_.instances(), 8);
+      BOOST_TEST_EQ(check_.constructions(), 10);
+
+      BOOST_TEST(r1.first == x.emplace_hint(r1.first,
+                               boost::unordered::piecewise_construct,
+                               boost::make_tuple(9, "", 'b', 4, 5, 6, 7, 8, 9),
+                               boost::make_tuple(15, "jkjk")));
+      BOOST_TEST(r1.first == x.emplace_hint(r2.first,
+                               boost::unordered::piecewise_construct,
+                               boost::make_tuple(9, "", 'b', 4, 5, 6, 7, 8, 9),
+                               boost::make_tuple(275, "xxx", 'm', 6)));
+      BOOST_TEST(r1.first == x.emplace_hint(x.end(),
+                               boost::unordered::piecewise_construct,
+                               boost::make_tuple(9, "", 'b', 4, 5, 6, 7, 8, 9),
+                               boost::make_tuple(-10, "blah blah", '\0')));
+      BOOST_TEST_EQ(x.size(), 2u);
+      BOOST_TEST(x.find(k2)->second == m2);
+      BOOST_TEST_EQ(check_.instances(), 8);
+      BOOST_TEST_EQ(check_.constructions(), 16);
+    }
 #endif
   }
 
+#ifdef BOOST_UNORDERED_FOA_TESTS
+  static boost::unordered_flat_map<emplace_value, emplace_value,
+    boost::hash<emplace_value> >* test_map;
+
+  static boost::unordered_node_map<emplace_value, emplace_value,
+    boost::hash<emplace_value> >* test_node_map;
+
+  UNORDERED_TEST(emplace_map, ((test_map)(test_node_map)))
+#else
+  static boost::unordered_map<emplace_value, emplace_value,
+    boost::hash<emplace_value> >* test_map;
+
+  UNORDERED_TEST(emplace_map, ((test_map)))
+#endif
+
 #ifndef BOOST_UNORDERED_FOA_TESTS
   UNORDERED_AUTO_TEST (emplace_multimap) {
     test::check_instances check_;
@@ -571,14 +672,13 @@ namespace emplace_tests {
   }
 #endif
 
-  UNORDERED_AUTO_TEST (try_emplace) {
+  template <class X> static void try_emplace(X*)
+  {
     test::check_instances check_;
-#ifdef BOOST_UNORDERED_FOA_TESTS
-    typedef boost::unordered_flat_map<int, emplace_value> container;
-#else
-    typedef boost::unordered_map<int, emplace_value> container;
-#endif
-    typedef container::iterator iterator;
+
+    typedef X container;
+
+    typedef typename container::iterator iterator;
     typedef std::pair<iterator, bool> return_type;
     container x(10);
     return_type r1, r2, r3;
@@ -615,12 +715,22 @@ namespace emplace_tests {
     BOOST_TEST_EQ(check_.constructions(), 4);
 
     BOOST_TEST(r2.first == x.try_emplace(r2.first, k2, 808709, "what"));
-    BOOST_TEST(
-      r2.first ==
-      x.try_emplace(r2.first, k2, 10, "xxx", 'a', 4, 5, 6, 7, 8, 9, 10));
+    BOOST_TEST(r2.first == x.try_emplace(r2.first, k2, 10, "xxx", 'a', 4, 5, 6,
+                             7, 8, 9, 10));
     BOOST_TEST(r2.first->second == m2);
     BOOST_TEST_EQ(x.size(), 2u);
   }
+
+#ifdef BOOST_UNORDERED_FOA_TESTS
+  static boost::unordered_flat_map<int, emplace_value>* test_int_map;
+  static boost::unordered_node_map<int, emplace_value>* test_int_node_map;
+
+  UNORDERED_TEST(try_emplace, ((test_int_map)(test_int_node_map)))
+#else
+  static boost::unordered_map<int, emplace_value>* test_int_map;
+
+  UNORDERED_TEST(try_emplace, ((test_int_map)))
+#endif
 }
 
 RUN_TESTS()
