@@ -8,8 +8,6 @@
 // Official repository: https://github.com/boostorg/url
 //
 
-#ifndef BOOST_URL_IMPL_SEGMENTS_ENCODED_VIEW_IPP
-#define BOOST_URL_IMPL_SEGMENTS_ENCODED_VIEW_IPP
 
 #include <boost/url/detail/config.hpp>
 #include <boost/url/segments_encoded_view.hpp>
@@ -44,4 +42,3 @@ segments_view() const noexcept
 } // urls
 } // boost
 
-#endif
